      SUBROUTINE chk_input
c     ============================================================
c     check for inconsistencies of correlated input parameter
c     such as MAGIN and MAGINFILE. Guerantee compability for 
c     older versions of GENESIS 1.3
c     ------------------------------------------------------------
c
      include 'genesis.def'
      include 'input.cmn'
c
      integer i,ix
      REAL*8  RW0
c
c     set seeds for random number generator to negative value    
c
      ISEED=-IABS(ISEED)
      IPSEED=-IABS(IPSEED)
      quadf=abs(quadf)
      quadd=-abs(quadd)
c
c     set flags to either 0 or 1 + adjust values 
c
      IF (ISRAVG.NE.0) ISRAVG=1
      IF (ISRSIG.NE.0) ISRSIG=1
      if (LBC.NE.0) LBC=1
      if (IORB.NE.0) IORB=1
      if (MAGIN.NE.0) MAGIN=1
      if (MAGOUT.NE.0) MAGOUT=1
      if (IDUMP.NE.0) IDUMP=1
      if (IDMPPAR.NE.0) IDMPPAR=1
      if (IDMPFLD.NE.0) IDMPFLD=1
      if (IOTAIL.NE.0) IOTAIL=1
      if (ILOG.ne.0) ILOG=1
      if (IALL.NE.0) IALL=1
      if (ITDP.NE.0) ITDP=1
      if (WCOEFZ(2).gt.1) WCOEFZ(2)=1.d0
      if (IWITYP.NE.0) IWITYP=1
      if (DELAW.LT.SMALL) IERTYP=0
      if (IERTYP.EQ.0) DELAW=0.d0
      if ((DGRID.LE.SMALL).and.(ZRAYL.GT.0.)) THEN  !grid size determined by beam size?
         RW0 = DSQRT((ZRAYL*XLAMDS/PI)*(1.0d0+(ZWAIST/ZRAYL)**2))
         DGRID = RMAX0*(RW0+dSQRT(RXBEAM**2+RYBEAM**2))/2.d0
      endif   
c
c     check for version specific changes
c
      if (idump.gt.0) then
         idmppar=idump
         if (version.ge.1.0) idmpfld=idump
      endif   
c
c     Check case if time dependent code is selected
c
      IF (ITDP.eq.0) THEN
        NSLICE=1                             !one slice 
        ZSEP=DELZ                            !one bucket
        NTAIL=0                              !middle of the beam
        CURLEN=-1.d0                         !step profile in z
        SHOTNOISE=0.d0                       !no phase fluctuation 
        IALL=1                               !reference loading for scan
        ISHSTY=1                             !make sure for output
        ISRADI=1                             ! -- " --
        ISPART=1                             ! -- " --
        IOTAIL=0                             !cut tails
      ENDIF
c
c     check for scan function 
c
      if (INDEX(SCAN,' ').NE.1) ISCAN=chk_scan(scan,iscan)
c
      if (iscan.gt.0) then
        if (NSCAN.LE.1) then
           i=PRINTERR(ERRINWARN,'NSCAN too small for scan') 
           ISCAN=0
        ELSE   
          if (((magin+magout).gt.0).and.((iscan.eq.5).or.(iscan.eq.6)
     c         .or.(iscan.eq.13).or.(iscan.eq.14))) then
            if (magin.ne.0) then
               i=PRINTERR(ERRSCANM1,MAGINFILE) 
               ISCAN=0
            else
               i=PRINTERR(ERRSCANM2,MAGOUTFILE)
               ISCAN=0
            endif   
          else
            NSLICE=NSCAN
            IALL=1
            ZSEP=DELZ
            CURLEN=-1.d0
            NTAIL=0
            IOTAIL=0
            SHOTNOISE=0.0
            if (itdp.ne.0) then
              i=PRINTERR(ERRSCANT,' ')
              itdp=0
            endif
          endif  
        endif  
      endif
c
      if (NSLICE.LT.1) NSLICE=1 
c
c     check for magnet field input & output 
c
      if (index(MAGINFILE,' ').ne.1) then
         magin=1
      else   
        if (magin.ne.0) then
          if (ILOG.NE.0) then     
            i=PRINTERR(ERRREQUEST,'MAGINFILE')   !interaction required
            call last
          endif
 1        write(6,100) 
          read(5,200) maginfile !get magnetic input file name
          if (INDEX(maginfile,' ').eq.1) then
             i=PRINTERR(ERRINVALNAME,maginfile)
             goto 1
          endif   
        endif
      endif 
      if (index(MAGOUTFILE,' ').ne.1) then
         magout=1
      else   
        if (magout.ne.0) then
           if (ILOG.NE.0) then     
             i=PRINTERR(ERRREQUEST,'MAGOUTFILE')   !interaction required
             call last
           endif
 2         write(6,110)
           read(5,200) magoutfile !get magnetic input file name
           if (INDEX(MAGOUTFILE,' ').EQ.1) then
              i=PRINTERR(ERRINVALNAME,magoutfile)
              goto 2
           endif   
        endif
      endif 
c
c     check for output parameters
c
      IF (ISHSTY.LT.1) ISHSTY=1
      IF (ISPART.LT.1) ISPART=1
      IF (ISRADI.LT.1) ISRADI=1
      IX=0
      DO i=1,14+NHMAX
         IF (LOUT(I).NE.0) THEN
            IX=IX+1
            LOUT(i)=1
         ENDIF   
      ENDDO
      IF (IX.EQ.0) IPHSTY=0
      RETURN
c
c     format statements
c
 100  Format('PLEASE ENTER MAGNETIC INPUT FILE NAME')
 110  Format('PLEASE ENTER MAGNETIC OUTPUT FILE NAME')
 200  FORMAT(a30)
c 
      END !chk_input

      FUNCTION CHK_BND()
c     ==================================================================
c     Checks some boundaries of the input file.
c     ------------------------------------------------------------------
c
      INCLUDE  'genesis.def'
      include  'input.cmn'
c
      INTEGER ITMP,IBAS(7),i1,i2,i
c
      CHK_BND=NOERR
      ITMP=0     
c

      If ((AW0.LE.0.d0).and.(MAGIN.eq.0)) Then
         itmp=PRINTERR(ERRINPUT,'No resonable wiggler field defined') !abort
      EndIf
      IF (NWIG.LE.0) then
         ITMP=PRINTERR(ERRINPUT,'NWIG must be positive and non-zero')
      endif
      IF (DELZ.LE.0) then
         ITMP=PRINTERR(ERRINPUT,'DELZ must be positive and non-zero')
      endif
      IF (ZSEP.LE.0) then
         ITMP=PRINTERR(ERRINPUT,'ZSEP must be positive and non-zero')
      endif
      IF (XLAMD.LE.0) THEN
         ITMP=PRINTERR(ERRINPUT,'XLAMD must be positive and non-zero')  !abort
      ENDIF        
      IF ((GAMMA0-4*DELGAM).LT.1) THEN
         ITMP=PRINTERR(ERRINPUT,'Energy GAMMA0 too small')  !abort
      ENDIF
      If(NPART.GT.NPMAX) Then
         I=PRINTERR(ERRINWARN,'NPART > NPMAX - Setting NPART=NPMAX')
      EndIf
      IF (NBINS.LT.4) THEN
        I=PRINTERR(ERRINWARN,'NBINS too small - Setting NBINS=4')
        NBINS=4
      endif  
      If(MOD(NPART,NBINS*4).NE.0) Then
         ITMP=PRINTERR(ERRINPUT,'NPART not a multiple of 4*NBINS') !abort
      EndIf
      IF (XLAMDS.LE.0) THEN
         ITMP=PRINTERR(ERRINPUT,'XLAMDS must be positive')  !abort
      ENDIF        
      IF (PRAD0.Lt.0.d0) THEN
         ITMP=PRINTERR(ERRINPUT,'PRAD0 must not be negative')!abort
      ENDIF
      IBAS(1)=ILDPSI
      IBAS(2)=ILDX
      IBAS(3)=ILDY
      IBAS(4)=ILDPX
      IBAS(5)=ILDPY
      IBAS(6)=ILDGAM
      IBAS(7)=ILDGAM+1
      DO i1=1,7
       DO i2=i1+1,7
        IF (IBAS(i1).EQ.IBAS(i2)) THEN !no abort
         i=PRINTERR(ERRINWARN,'Identical bases in Hammersley sequences')
        ENDIF
       ENDDO
      ENDDO   
      If ((ABS(IERTYP).NE.0).AND.(ABS(DELZ-0.5D0).GT.SMALL)) Then
         ITMP=PRINTERR(ERRINPUT,'DELZ must be 0.5 for field errors')
      EndIf
      If (ISCAN.GT.22) THEN
        i=PRINTERR(ERRINWARN,'Invalid Scan parameter - Setting ISCAN=0')
        iscan=0
      ENDIF
      IF ((ISCAN.GT.0).AND.((NSCAN.LE.1).OR.(abs(SVAR).LT.SMALL))) THEN
         ITMP=PRINTERR(ERRINPUT,'Invalid scan range (NSCAN,SVAL)')
      ENDIF
      If ((FL/DELZ-1.*INT(FL/DELZ)).GT.SMALL) THEN
         ITMP=PRINTERR(ERRINPUT,'FL not a multiple of DELZ')
      ENDIF   
      If ((DL/DELZ-1.*INT(DL/DELZ)).GT.SMALL) THEN
         ITMP=PRINTERR(ERRINPUT,'DL not a multiple of DELZ')
      ENDIF   
      If ((DRL/DELZ-1.*INT(DRL/DELZ)).GT.SMALL) THEN
         ITMP=PRINTERR(ERRINPUT,'DRL not a multiple of DELZ')
      Endif 
      If ((F1ST/DELZ-1.*INT(F1ST/DELZ)).GT.SMALL) THEN
         ITMP=PRINTERR(ERRINPUT,'F1ST not a multiple of DELZ')
      endif   
      If ((ZSEP/DELZ-1.*INT(ZSEP/DELZ)).GT.SMALL) THEN
         ITMP=PRINTERR(ERRINPUT,'ZSEP not a multiple of DELZ')
      Endif 
      If ((SL/DELZ-1.*INT(SL/DELZ)).GT.SMALL) THEN
         ITMP=PRINTERR(ERRINPUT,'SL not a multiple of DELZ')
      endif
      IF (NSLICE.GT.NSMAX) THEN
         ITMP=PRINTERR(ERRINPUT,'Too many slices (NSLICE>NSMAX)')
      ENDIF
      IF (ZRAYL.LE.0.) then
         ITMP=PRINTERR(ERRINPUT,'ZRAYL must be larger than 0')
      ENDIF
      IF(MOD(NCAR,2).EQ.0) Then
         ITMP=PRINTERR(ERRINPUT,'NCAR not an odd integer') 
      EndIf                
      IF(NCAR.GT.NCMAX) Then
         i=PRINTERR(ERRINWARN,'NCAR too large - setting NCAR=NCMAX')
      EndIf                
      IF (NPTR.GT.NRGRID-1) THEN
         I=PRINTERR(ERRINWARN,'NPTR too large - setting NPTR=NRGRID')
      ENDIF
      IF (NPTR.LT.2) THEN
        I=PRINTERR(ERRINWARN,'NPTR too small - disabling space charge')
        NSCZ=0
        NSCR=0
      ENDIF
      IF (NSCZ.GT.(NBINS/2+1)) THEN  !somehow empirical boundary
        i=PRINTERR(ERRINWARN,'NSCZ too large - setting NSCZ=3')
        NSCZ=3
      ENDIF        
      IF (NHARM.GT.(NBINS/2+1)) THEN
        i=PRINTERR(ERRINWARN,'Higher harmonics are inaccurate (NHARM)')
      ENDIF
c
      chk_bnd=itmp
      Return
      End     !CHK_BND
c
      FUNCTION chk_scan(c0,iscn)
c     ============================================================
c     Check for string in input SCAN
c     ------------------------------------------------------------
c
      character*30 c0
      integer i,j,iscn,chk_scan 
c
      chk_scan=iscn          !if not found use value if iscan
      CALL TOUPPERCASE(c0)
      j=1
      Do i=1,30               !conversion to UPPERCASE 
         IF (c0(i:i).Gt.' ') j=i    !end of string?
      ENDDO

      IF (c0(1:j).Eq.'GAMMA0')  chk_scan=1
      IF (c0(1:j).Eq.'DELGAM')  chk_scan=2
      IF (c0(1:j).Eq.'CURPEAK') chk_scan=3
      IF (c0(1:j).Eq.'XLAMDS')  chk_scan=4
      IF (c0(1:j).Eq.'AW0')     chk_scan=5
      IF (c0(1:j).Eq.'ISEED')   chk_scan=6
      IF (c0(1:j).Eq.'PXBEAM')  chk_scan=7
      IF (c0(1:j).Eq.'PYBEAM')  chk_scan=8
      IF (c0(1:j).Eq.'XBEAM')   chk_scan=9
      IF (c0(1:j).Eq.'YBEAM')   chk_scan=10
      IF (c0(1:j).Eq.'RXBEAM')  chk_scan=11
      IF (c0(1:j).Eq.'RYBEAM')  chk_scan=12
      IF (c0(1:j).Eq.'XLAMD')   chk_scan=13
      IF (c0(1:j).Eq.'DELAW')   chk_scan=14
      IF (c0(1:j).Eq.'ALPHAX')  chk_scan=15
      IF (c0(1:j).Eq.'ALPHAY')  chk_scan=16
      IF (c0(1:j).Eq.'EMITX')   chk_scan=17
      IF (c0(1:j).Eq.'EMITY')   chk_scan=18
      IF (c0(1:j).Eq.'PRAD0')   chk_scan=19
      IF (c0(1:j).Eq.'ZRAYL')   chk_scan=20
      IF (c0(1:j).Eq.'ZWAIST')  chk_scan=21
      IF (c0(1:j).EQ.'AWD')     chk_scan=22
      
      RETURN
      END
