      Subroutine DIAGNO(ISTEPZ,XKPER0)
c     ==================================================================
c     Some Diagnostics:
c     The radiation power must be calculated for each integration step
c     otherwise ERROR will be wrong.
c     All calculation are stored in a history arrays which will be
c     written to a file ad the end of the run.
c     ------------------------------------------------------------------
c
      INCLUDE  'genesis.def'
      INCLUDE  'input.cmn'
      INCLUDE  'field.cmn'
      INCLUDE  'particle.cmn'
      INCLUDE  'diagnostic.cmn'
      INCLUDE  'work.cmn'
      INCLUDE  'magnet.cmn'    ! unofficial
c
      INTEGER i,ip,ix,iy,i0,i1,nn(2),istepz
      REAL*8 xavg,yavg,tpsin,tpcos,prad,ptot,gainavg,
     +       xxsum,yysum,cr2,crsum,wwcr,xkper0
      COMPLEX*8 CTMP
c
      IF (IPHSTY.le.0) RETURN      !no output at all
c
c     DIAGNOSTIC: radiation field
c     -----------------------------------------------------------------
c     
c     Radiation power
c
      CRSUM=0.0d0
      ctmp=DCMPLX(0.,0.)
      DO I=1,NCAR*NCAR
         WWCR=DBLE(CRFIELD(I)*CONJG(CRFIELD(I))) !=sum of |Aij|^2 
         CRSUM=CRSUM+WWCR
         ctmp=ctmp+CRFIELD(i)
      END DO
      PRAD=CRSUM*(DXY*EEV*XKPER0/XKS)**2/vacimp
      if (PRADOL.gt.0.d0) then
        LOGP=DLOG(PRAD/PRADOL)/(DELZ*XLAMD) !Averaged Energy in units of gamma
      else
        logp=0.
      endif 
      GAINAVG=0.5D0*(PRAD+PRADOL)            !Average with old power (leap frog!)
      gain=PRAD
      PRADOL=PRAD                                 !store actual value as old value
      PTOT=PRAD 
      FFIELD=DBLE(Ctmp*CONJG(ctmp)) ! Far field on-axis (a.u.)   

c
      IF (MOD(ISTEPZ,IPHSTY).NE.0) RETURN   
c
c     Power and Phase of center point
c
      I=NCAR*(NCAR-1)/2+(NCAR+1)/2
      POWMID=DBLE(CRFIELD(I)*CONJG(CRFIELD(I)))
      POWMID=POWMID*(DXY*EEV*XKPER0/XKS)**2/vacimp
      PHIMID=DATAN2(DIMAG(CRFIELD(I)),DBLE(CRFIELD(I)))
      REALFIELDMID=DREAL(CRFIELD(I))
      IMAGFIELDMID=DIMAG(CRFIELD(I))
c
c     Radiation size  
c
      if (prad.gt.0.d0) then
        XAVG=0.0d0
        YAVG=0.0d0
        CR2=0.0d0
        DO IY=1,NCAR
          DO IX=1,NCAR
            I=(IY-1)*NCAR+IX
            WWCR=DBLE(CRFIELD(I)*CONJG(CRFIELD(I)))
            XAVG=XAVG+WWCR*DBLE(ix)                      !sum up position
            YAVG=YAVG+WWCR*DBLE(iy)
            CR2=CR2+WWCR*DBLE(IX*IX+IY*IY)
          END DO
        END DO
        XAVG=XAVG/CRSUM                                 !center of radiation
        YAVG=YAVG/CRSUM
        CR2=CR2/CRSUM-XAVG*XAVG-YAVG*YAVG
      else 
        CR2=0.d0
      endif
      WHALF=DSQRT(CR2)*DXY/XKPER0     !rms radiation size
c
c     DIAGNOSTIC: electron beam
c     ------------------------------------------------------------------
c
c     Energy
c
      GAMHIST=0.
      DGAMHIST=0.
      DO IP=1,NPART
         GAMHIST=GAMHIST+(GAMMA(IP)-GAMMA0) 
         DGAMHIST=DGAMHIST+(GAMMA(IP)-GAMMA0)**2 
      ENDDO
      GAMHIST=GAMHIST/DBLE(NPART)
      DGAMHIST=SQRT(DGAMHIST/DBLE(NPART)-GAMHIST**2)
c
      PTOT=PTOT+EEV * XCUREN * GAMHIST           !1st part of total energy      
      IF (ISTEPZ.EQ.0) PINIT=PTOT
c
c     Bunching at NHARM harmonics
c
      DO i=1,NHARM
        TPSIN=0.0d0                              
        TPCOS=0.0d0
        Do IP=1,NPART
          TPSIN=TPSIN+DSIN(THETA(IP)*DBLE(i))          !add up phases
          TPCOS=TPCOS+DCOS(THETA(IP)*DBLE(i))
        END DO   !IP
        TPSIN=TPSIN/DBLE(NPART)     
        TPCOS=TPCOS/DBLE(NPART)
        PMODHIST(i)=DSQRT(TPSIN**2+TPCOS**2) !Bunching factor
        REALPMODHIST(i) = TPCOS
        IMAGPMODHIST(i) = TPSIN
      ENDDO
      DO i=NHARM+1,NHMAX
        PMODHIST(i)=0.
        REALPMODHIST(i) = 0.
        IMAGPMODHIST(i) = 0.
      ENDDO 
c
c     Beam Radius & energy spread
c
      XXSUM=0.0d0    !Reset counter
      YYSUM=0.0d0
      XPOS=0.0d0   
      YPOS=0.0d0
c     
      Do I=1,NPART
         XXSUM=XXSUM+XPART(I)**2     !Sum of radii squared
         YYSUM=YYSUM+YPART(I)**2
         XPOS=XPOS+XPART(I)          !Sum of radii 
         YPOS=YPOS+YPART(I)
      End Do       ! I
      XPOS=XPOS/DBLE(NPART)/XKPER0     !mean value
      YPOS=YPOS/DBLE(NPART)/XKPER0     
      XXSUM=XXSUM/DBLE(NPART)/XKPER0/XKPER0    !mean square value
      YYSUM=YYSUM/DBLE(NPART)/XKPER0/XKPER0    
      XRMS=DSQRT(XXSUM-XPOS**2)
      YRMS=DSQRT(YYSUM-YPOS**2)
c      ypos=awz(istepz)   !unofficial
c
c     Energy conservation
c     ----------------------------------------------------------------
c
      ERROR=0.
      IF (GAINAVG.NE.0.0) THEN
        ERROR=100.d0*(ptot/GAINAVG-pinit/GAINAVG)
      ENDIF   
c
      IF (LOUT(6).EQ.0) RETURN

c 
c     Diffraction Angle of Radiation Field
c     ---------------------------------------------------------------
c
      NCTMP=2**(INT(log(float(NCAR))/log(2.))+1)   !with NCTMP> ncar
      DO i1=1,NCTMP*NCTMP
         CRWORK3(i1)=(0.d0,0.d0)                   !clear working
      ENDDO
      i=(NCTMP-NCAR)/2                             !first index in bigger mesh
      DO ix=1,NCAR
         DO iy=1,NCAR
            I0=(ix-1)*NCAR+iy
            I1=(ix-1+i)*NCTMP+iy+i
            CRWORK3(I1)=CRFIELD(I0)                !copy field around mid point 
         ENDDO
      ENDDO
      nn(1)=nctmp                                  !size of mesh
      nn(2)=nctmp
c
c     debug
c
c      CRSUM=0.0
c      do i=1,ncar*ncar
c         crsum=crsum+DBLE(CRWORK3(i)*CONJG(CRWORK3(i)))
c      enddo
c      write (*,*) 'field power before FFT',crsum

      CALL FOURN(CRWORK3,nn,2,1)                   !2D FFT with complex variables 
c
c     debug
c
c      CRSUM=0.0
c      do i=1,ncar*ncar
c         crsum=crsum+DBLE(CRWORK3(i)*CONJG(CRWORK3(i)))
c      enddo
c      write (*,*) 'field power before FFT',crsum

c      pause
c
      DO ix=1,NCTMP/2                              !rearrange FFT output
         DO iy=1,NCTMP/2
            I0=(ix-1)*NCTMP+iy
            I1=(ix-1+NCTMP/2)*NCTMP+iy+NCTMP/2
            CTMP=CRWORK3(I1) 
            CRWORK3(I1)=CRWORK3(I0) 
            CRWORK3(I0)=CTMP 
            I0=(ix-1)*NCTMP+iy+NCTMP/2
            I1=(ix+NCTMP/2-1)*NCTMP+iy 
            CTMP=CRWORK3(I1) 
            CRWORK3(I1)=CRWORK3(I0) 
            CRWORK3(I0)=CTMP 
         ENDDO
      ENDDO
c
      XAVG=0.0d0
      YAVG=0.0d0
      CRSUM=0.0d0
      DO IY=1,NCTMP
        DO IX=1,NCTMP
          I=(IY-1)*NCTMP+IX
          WWCR=DBLE(CRWORK3(I)*CONJG(CRWORK3(I)))
          XAVG=XAVG+WWCR*DBLE(ix)                      !sum up spatial frequency
          YAVG=YAVG+WWCR*DBLE(iy)
          CRSUM=CRSUM+WWCR
        END DO
      END DO
      if (CRSUM.GT.0.0) then
        XAVG=XAVG/CRSUM                                  !center of spatial frequency
        YAVG=YAVG/CRSUM
      else
        XAVG=0.
        YAVG=0.
      endif
c
      CR2=0.0d0
      DO IY=1,NCTMP
         DO IX=1,NCTMP
            I=(IY-1)*NCTMP+IX
            WWCR=DBLE(CRWORK3(I)*CONJG(CRWORK3(I)))
            CR2=CR2+WWCR*((DBLE(ix)-XAVG)**2+(DBLE(iy)-YAVG)**2)
         END DO
      END DO
c      
      if (CRSUM.le.0.0) then
         CR2=0.
         CRSUM=1.
      endif
c
      DIVER=DSQRT(CR2/CRSUM)*XKPER0*XLAMDS/DXY/DBLE(NCTMP)  !rms divergence angle
c
      RETURN
      END


