      Subroutine ESOURCE(ISTEPZ)
c     ==================================================================
c     Calculates the space charge field.
c     All particle are discretized on a radial mesh for all
c     selected azimutal and longitudinal fourier modes.
c     ------------------------------------------------------------------
c
      INCLUDE  'genesis.def'
      INCLUDE  'input.cmn'
      INCLUDE  'particle.cmn'
      INCLUDE  'work.cmn'
c
      INTEGER m,j,ip,ir,istepz
c
      COMPLEX *16 VN,COEF,CMA(NRGRID),CMB(NRGRID),CMC(NRGRID),
     +            CRTMP1(NRGRID),CRTMP2(NRGRID),CSCSOURCE(NRGRID)
      REAL*8 econst,RSCMAX,DRSC,VOL(NRGRID),XMID,YMID,xks,xkw0,
     +       RMID(NRGRID),RLOG(NRGRID),RDIG(NRGRID)      
c
      IF (NSCZ.LE.0) RETURN                       !No space charge selected
c
      xks=twopi/xlamds
      xkw0=twopi/xlamd

      XMID=0.0d0              !get centroid position   
      YMID=0.0d0
      Do Ip=1,NPART
         XMID=XMID+XPART(Ip)       
         YMID=YMID+YPART(Ip)
      End Do       ! I
      XMID=XMID/DBLE(NPART)   !mean value
      YMID=YMID/DBLE(NPART)     
c      
      RSCMAX=0.                                   !clear max radius
      DO ip=1,NPART 
        EZ(IP)=0.d0                               !clear old space charge field
        P1(IP)=DSQRT((XPART(IP)-XMID)**2+(YPART(IP)-YMID)**2)/XKw0   !get radius (in m)
        IF (P1(IP).GT.RSCMAX) RSCMAX=P1(IP)       !look for maximum radius 
        IF (P1(IP).eq.0.) THEN
           CPART1(IP)=DCMPLX(1.d0,0.d0)           !particle in origin
        ELSE 
           CPART1(IP)=DCMPLX(YPART(IP)-YMID,-XPART(IP)+XMID)/P1(IP)  !azimutal phase
        END IF
      END DO
c
      RSCMAX=2.*RSCMAX                            !maximum of radial mesh
      DRSC=RSCMAX/float(NPTR-1)                   !gridpoint distance
      DO IR=2,NPTR
        VOL(IR)=PI*DRSC*DRSC*(2.*IR-1)            !2D volume around grid point
        RLOG(IR)=LOG(float(IR)/float(IR-1))       !log term of higher modes
        RDIG(IR)=2.*PI*FLOAT(IR-1)                !diagonal terms above/below main diag. 
      END DO
      VOL(1)=PI*DRSC*DRSC                         !volume of origin            
      RLOG(1)=LOG(0.5)                            !shielding radius
      RDIG(1)=0.                                  !no lower element at origin
      RDIG(NPTR+1)=0.                             !no upper element at border 
c
      ECONST=VACIMP/EEV*XCUREN/FLOAT(NPART)/(XKS+XKW0)  !source term normalization
      COEF=DCMPLX(1.d0/(XKS**2-(XKS+XKW0)**2),0.d0) !matrix normalisation 
c
      DO M=-NSCR,NSCR                             !count over azimutal modes          
c
         DO IP=1,NPART                            !get right azimutal component
            CPART2(IP)=CPART1(IP)**M
         END DO     
c
         DO IR=1,NPTR
           RMID(IR)=-RDIG(IR)-RDIG(IR+1)-2.*PI*FLOAT(M*M)*RLOG(IR) !main diagonal elements 
         END DO
         RMID(NPTR)=RMID(NPTR)-2.*PI*FLOAT(NPTR)     !Direchlet Boundary Condition
c         
         DO J=1,NSCZ 
            DO IR=1,NPTR
               CSCSOURCE(IR)=DCMPLX(0.d0,0.d0)      !clear source  
            END DO
            DO IP=1,NPART
               IR=INT(P1(IP)/DRSC)+1               !radial position
               CSCSOURCE(IR)=CSCSOURCE(IR)+CPART2(IP)* !add long and azimutal phase
     +            DCMPLX(DCOS(THETA(IP)*J),-DSIN(THETA(IP)*J))   
            ENDDO 
            DO IR=1,NPTR 
               VN=DCMPLX(0.d0,ECONST/FLOAT(j)/VOL(IR)) !complex norm. term
               CSCSOURCE(IR)=VN*CSCSOURCE(IR)            !scale source term (current density)
               CMA(IR)=COEF*DCMPLX(RDIG(IR)/J/J/VOL(IR),0.d0)  !construct complex matrix
              CMB(IR)=(1.D0,0.d0)+COEF*DCMPLX(RMID(IR)/J/J/VOL(IR),0.d0)
               CMC(IR)=COEF*DCMPLX(RDIG(IR+1)/J/J/VOL(IR),0.d0)
            ENDDO 
c            
            CALL TRIRAD(CMA,CMB,CMC,CSCSOURCE,CRTMP1,CRTMP2,NPTR) !solve equation
c
            DO IP=1,NPART                         !sum up fourier coefficient
               IR=INT(P1(IP)/DRSC)+1 
               EZ(IP)=EZ(IP)+2.*DBLE(CRTMP1(IR)/CPART2(IP)*
     +              DCMPLX(DCOS(THETA(IP)*J),DSIN(THETA(IP)*J)))   
            END DO
        END DO
      END DO
      DO IP=1,NPART
         EZ(IP)=EZ(IP)/xkw0           !scale due to normalized Z
      ENDDO
      RETURN
      END 




      Subroutine TRIRAD(A,B,C,R,U,W,N)
c     ==================================================================
c     Solve a tridiagonal system for radial mesh
c     Only called by ESOURCE for space charge calculation
c     ------------------------------------------------------------------
c
      INTEGER n,k
      COMPLEX*16 W(*),A(*),B(*),C(*),R(*),U(*),BET
c

      BET=B(1)
      U(1)=R(1)/BET
      Do K=2,N
	 W(K)=C(K-1)/BET
	 BET=B(K)-A(K)*W(K)
	 U(K)=(R(K)-A(K)*U(K-1))/BET
      End Do       ! K
      Do K=N-1,1,-1
	 U(K)=U(K)-W(K+1)*U(K+1)
      End Do       ! K
c
      Return
      End     !TRIRAD
