      Subroutine FIELD
c     ==================================================================
c     Integrate the wave equation one step in z for cartesian mesh
c     using ADI - methode (Alternationg Direction Implicit)
c        1: u(n+1/2)=u(n)+alpha/2(dx u(n+1/2)+dy u(n))
c        2: u(n+1)=u(n+1/2)+alpha/2(dx u(n+1/2)+dy u(n+1))  
c        To use TRIDIAG methode transpose data array:
c        (1)->u(n)->u(i,j)->u(j,i)->u(n)->(2)->transpose again 
c     ------------------------------------------------------------------
c
      INCLUDE   'genesis.def'
      INCLUDE   'field.cmn'
      include   'input.cmn'

      Integer ix,idx
c
c     homogenious part to right hand side of diff equation (1)
c     ------------------------------------------------------------------
c
      Do ix=1,ncar
        CRHM(ix)=CRSOURCE(ix)+CRFIELD(ix)
     +           +CSTEP*(CRFIELD(ix+NCAR)-CRFIELD(Ix)-CRFIELD(ix))
      End DO
      DO idx=NCAR+1,NCAR*(NCAR-1)
          CRHM(idx)=CRSOURCE(idx) 
     +              +CRFIELD(idx)+CSTEP*(CRFIELD(idx+NCAR)
     +              +CRFIELD(idx-ncar)-CRFIELD(idx)-CRFIELD(idx))
      END DO
      Do idx=ncar*(ncar-1)+1,NCAR*NCAR
        CRHM(idx)=CRSOURCE(idx)+CRFIELD(idx)
     +           +CSTEP*(CRFIELD(idx-NCAR)-CRFIELD(idx)-CRFIELD(IDX))
      End DO
c
c     Neumann boundary condition
c     ------------------------------------------------------------------
c
      IF (LBC.NE.0) THEN
         idx=NCAR*(NCAR-1)
         DO ix=1,NCAR
            CRHM(ix)=CRHM(ix)+CSTEP*CRFIELD(ix+NCAR)
            CRHM(IDX+IX)=CRHM(IDX+IX)+CSTEP*CRFIELD(IDx+IX-NCAR)
         ENDDO
      ENDIF
c
c     Solve the tridiagonal system 1
c     ------------------------------------------------------------------
c
      CALL TRIDAGX(CRMATC,CRHM,CRFIELD)
c
c     homogenious part to right hand side of diff equation (2)
c     ------------------------------------------------------------------
c
c
      DO ix=1,NCAR*(NCAR-1)+1,NCAR
         CRHM(ix)=CRSOURCE(IX)+CRFIELD(ix)
     +           +CSTEP*(CRFIELD(ix+1)-CRFIELD(Ix)-CRFIELD(ix))
         DO idx=ix+1,ix+NCAR-2
            CRHM(idx)=CRSOURCE(idx)
     +                +CRFIELD(idx)+CSTEP*(CRFIELD(idx+1)
     +                +CRFIELD(idx-1)-CRFIELD(idx)-CRFIELD(idx))
         ENDDO
         idx=ix+NCAR-1
         CRHM(idx)=CRSOURCE(IdX)+CRFIELD(idx)
     +           +CSTEP*(CRFIELD(idx-1)-CRFIELD(Idx)-CRFIELD(idx))
      ENDDO 

c
c     Neumann boundary condition
c     ------------------------------------------------------------------
c
      IF (LBC.NE.0) THEN
         DO ix=1,NCAR
            idx=NCAR*(IX-1)+1
            CRHM(idx)=CRHM(idx)+CSTEP*CRFIELD(idx+1)
            idx=idx+NCAR-1
            CRHM(IDX)=CRHM(IDX)+CSTEP*CRFIELD(IDx-1)
         ENDDO
      ENDIF
c
c     Solve the tridiagonal system 2
c     ------------------------------------------------------------------
c
      CALL TRIDAGY(CRMATC,CRHM,CRFIELD)
c
      Return
      End     !FIELDCAR
c
c
      Subroutine Tridagx(C,R,U)
C     ==================================================================
C     Solve A Tridiagonal System For Cartesian Mesh in x direction
C     CBET and CWET are precalculated in AUXVAL
C     ------------------------------------------------------------------
C
      Include 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'field.cmn'

      Integer K,I
      Complex*16 C(*),R(*),U(*)
C

      DO I=0,NCAR*(NCAR-1),NCAR
        U(I+1)=R(I+1)*CBET(1)
        Do K=2,NCAR
	  U(K+I)=(R(K+I)-C(K)*U(K+I-1))*CBET(K)
        End Do       ! K
        DO K=NCAR-1,1,-1
          U(K+I)=U(K+I)-CWET(K+1)*U(K+I+1) 
        END DO
      END DO
C
      Return
      End     !Tridag
c
c
      Subroutine Tridagy(C,R,U)
C     ==================================================================
C     Solve A Tridiagonal System For Cartesian Mesh in y direction
C     CBET and CWET are precalculated in AUXVAL
C     ------------------------------------------------------------------
C
      Include 'genesis.def'
      include 'field.cmn'
      include 'input.cmn'

      Integer N,K,I
      Complex*16 C(*),R(*),U(*)
C
      DO I=1,NCAR
         U(I)=R(I)*CBET(1)
      ENDDO
      DO K=2,NCAR
         N=K*NCAR-NCAR
         DO I=1,NCAR
            U(N+I)=(R(N+I)-C(K)*U(N+I-NCAR))*CBET(K)
         ENDDO
      ENDDO
      DO K=NCAR-1,1,-1
         N=K*NCAR-NCAR 
         DO I=1,NCAR
            U(N+I)=U(N+I)-CWET(K+1)*U(N+I+NCAR)
         ENDDO
      ENDDO
C
      Return
      End     !Tridag
c
c
c   
      SUBROUTINE GETDIAG(STEPSIZE,GRIDSIZE,WAVENUMBER)
c     ======================================================================
c     Construct the diagonal matrix for field equation
c     Do some precalculation for field solver
c     ----------------------------------------------------------------------
c
      include 'genesis.def'
      include 'input.cmn'
      INCLUDE 'field.cmn'
      INCLUDE 'work.cmn'
c
      INTEGER icar,ix
      REAL*8 MUPP,MMID,MLOW,rtmp,STEPSIZE,GRIDSIZE,WAVENUMBER 
      COMPLEX*16  CWRK1(NCMAX),CWRK2(NCMAX)
      DIMENSION MUPP(NCMAX),MMID(NCMAX),MLOW(NCMAX)
c
c     Construction of the diagonal maxtrix for cartesian mesh
c     --------------------------------------------------------------------
      rtmp=0.25d0*STEPSIZE/(WAVENUMBER*GRIDSIZE*GRIDSIZE)  !factor dz/(4 Ks dx^2)
      CSTEP=dcmplx(0.d0,rtmp)    !complex value - see field equation  
      IF (LBC.NE.0) LBC=1        !boundary condition
      MUPP(1)=rtmp         !one edge of mesh 
      MMID(1)=-dble(2-LBC)*rtmp   !boundary condition A=0 or dA/dz=0 
      MLOW(1)=0.d0          
      DO ix=2,ncar-1
         MUPP(ix)=rtmp        !inside of mesh -> 2nd derivation possible
         MMID(ix)=-2.d0*rtmp
         MLOW(ix)=rtmp
      END DO
      MUPP(ncar)=0.d0       !other edge of mesh
      MMID(ncar)=-dble(2-LBC)*rtmp
      MLOW(ncar)=rtmp
c
c     Construct Complex Matrix CRMAT=(I-iM) for
c     field equation  (I-iM)*A(t+1)=(I+iM)A(t)
c     -------------------------------------------------------------------------
      DO icar=1,ncar
        CWRK1(icar)=-dcmplx(0.d0,MUPP(icar))            !store value temporary in CWRK1
        cwrk2(icar)=(1.d0,0.d0)-dcmplx(0.d0,MMID(icar)) !same here
        CRMATC(icar)=-dcmplx(0.d0,MLOW(icar))            !CRMATC is used later in the code
      END DO
c                                         
c     Precalculated constants for TRIDIAG subroutine
c     ------------------------------------------------------------------------ 
      CBET(1)=1./cwrk2(1)
      CWET(1)=0.
      DO icar=2,ncar
         CWET(icar)=CWRK1(icar-1)*CBET(icar-1)
         CBET(icar)=1./(cwrk2(icar)-CRMATC(icar)*CWET(icar))
      END DO
      Return
      End     !AUXVAL
c
