      SUBROUTINE SEED
c     =========================================================
c     Fills the Array CRTIME with a seeding field for the first 
c     slice.
c     ---------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'timerec.cmn'
      INCLUDE 'field.cmn'
      INCLUDE 'sim.cmn'
      INCLUDE 'work.cmn'
      INCLUDE 'io.cmn'
      INCLUDE 'time.cmn'
      include 'sdds.cmn'
      INCLUDE "SDDSFortran.h"
c
      COMPLEX*16 CGAUSS
      REAL*8  ZSCAL,XCR,YCR,RCR2
      INTEGER IX,IY,IDX,J
c
      INTEGER i,k,pages,rows
      DIMENSION column1(NCAR*NCAR), column2(NCAR*NCAR)
      REAL column1, column2
c
c     OPTICAL BEAM PROFILE - gaussian distributed
c     ------------------------------------------------------------------
c
      IF (ITDP.EQ.0) RETURN
c
c     Seding of the random number generator
c     ----------------------------------------------------------------
c
      IPSEED=-ABS(IPSEED)      
      XCR=RAN1(IPSEED)        !dummy call of RAN-function for seeding
c
c

      CGAUSS=0.5d0*XKS/XKPER0**2/DCMPLX(ZRAYL,-ZWAIST) !=ks/q see Siegman 
      ZSCAL = dSQRT(2.d0*VACIMP*PRAD0/PI * dble(CGAUSS) )!amplitude
     +        *XKS/XKPER0/EEV !?
      DO IY=1,NCAR
         DO IX=1,NCAR
           idx=(iy-1)*ncar+ix
           XCR=DXY*float(IX-1)-DGRID*xkw0
           YCR=DXY*float(IY-1)-DGRID*xkw0
           RCR2=XCR*XCR+YCR*YCR 
           CRWORK3(idx)=ZSCAL*CDEXP(-CGAUSS*RCR2)          !gaussian beam
         End Do    !IX
      End DO       !IY
c
c     Load seeding from file
c     -----------------------------------------------------------
c
      IF (NFIN.GT.0) THEN
         If (NOFILE.EQ.0) THEN 
            DO IY=1,NCAR*NCAR  
               CRTIME(IY+(NSLP-1)*NCAR*NCAR)=CRWORK3(IY)
            ENDDO
         ELSE
            WRITE(NTMP,REC=NSLP) (CRWORK3(IY),IY=1,NCAR*NCAR)
         ENDIF
c
         DO j=NSLP-1,1,-1
            if (SDDS .EQ. 0) then
               READ(NFIN,REC=j,ERR=1) (CRWORK2(IY),IY=1,NCAR*NCAR)
            else
               pages = j - SDDSNFINCurrentPage
               if (pages .LT. 0) then
                  write (6,*) "Problem reading field file"
                  stop 1
               end if
               do k = 1, pages
                  i = SDDS_ReadPage_F(1)
                  if (i .EQ. 0) then
                     call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
                     stop 1
                  end if
                  if (i .EQ. -1) then
                     call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
                     stop 1
                  end if
               end do
               SDDSNFINCurrentPage = SDDSNFINCurrentPage + pages
               rows = SDDS_RowCount_F(1)
               if (rows .NE. NCAR*NCAR) then
                  write (6,*) "invalid number of rows in the field file"
                  stop 1
               end if
               i = SDDS_GetColumnData_F(1,SDDSNFINRealIndex,column1,
     +              rows,SDDS_FLOAT)
               if (i .LT. 0) then
                  call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
                  stop 1
               end if
               i = SDDS_GetColumnData_F(1,SDDSNFINImagIndex,column2,
     +              rows,SDDS_FLOAT)
               if (i .LT. 0) then
                  call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
                  stop 1
               end if
               do i = 1,rows
                  CRWORK2(i) = CMPLX(column1(i),column2(i))
               end do
            endif
            IF (NOFILE.EQ.0) THEN
               DO iy=1,ncar*ncar
                  CRTIME((NSLP-1-j)*NCAR*NCAR+IY)=CRWORK2(IY)
               ENDDO 
            ELSE
               DO iy=1,ncar*ncar
                  CRWORK3(IY)=CRWORK2(IY)
               ENDDO 
               WRITE(NTMP,REC=NSLP-j,ERR=1) (CRWORK3(IY),IY=1,NCAR*NCAR)  
            ENDIF   
         ENDDO
         RETURN
 1       NFIN=-1
         WRITE(*,*)
         WRITE(*,*) '***** ERROR: FILE ERROR'
         WRITE(*,*) '***** Using internal field seeding'
         Write(*,*)
         DO IY=1,NCAR
            DO IX=1,NCAR
              idx=(iy-1)*ncar+ix
              XCR=DXY*float(IX-1)-DGRID*xkw0
              YCR=DXY*float(IY-1)-DGRID*xkw0
              RCR2=XCR*XCR+YCR*YCR 
              CRWORK3(idx)=ZSCAL*CDEXP(-CGAUSS*RCR2)          !gaussian beam
            End Do    !IX
         End DO       !IY
      ENDIF   
c 
      DO IX=0,NSLP-1
        IF (NOFILE.EQ.0) THEN
           DO IY=1,NCAR*NCAR                   
             CRTIME(IY+IX*NCAR*NCAR)=CRWORK3(IY)              !use always same seed
           END DO
        ELSE
           WRITE(NTMP,REC=IX+1) (CRWORK3(IY),IY=1,NCAR*NCAR)
        ENDIF
      END DO
c     
      RETURN
      END      !of SEED
 
