      SUBROUTINE INITIO
c     ==================================================================
c     Manages all initial input for genesis
c     ------------------------------------------------------------------

      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'io.cmn'

      INTEGER ierr1,ierr2,i
      character*34 file
c
      NLOG=6
      call first
      ierr1=READIN()                     !read namelist
c
      if (ilog.ne.0) then
        If (INDEX(OUTPUTFILE,' ').eq.1) then
         FILE='log-file'
         goto 5
        ELSE   
         FILE=OUTPUTFILE(1:STRLEN(OUTPUTFILE))//'.log'
        endif   
        OPEN(16,FILE=file,status='unknown',ERR=5)
        NLOG=16
 5      if (NLOG.eq.6)  i=PRINTERR(ERROPEN,file)
      endif
c
      ierr2=OPENOUTPUTFILE(ierr1)        !open outputfile+dump namelist
      if ((ierr2+ierr1).LT.0) call LAST  !error occured?
c
      call chk_input                     !make input consistent
c
c     open addition output files
c
      if (IPRADI.GT.0) then !is output of radiation field selected?
         IRECFLD=1
         NFLD =openbinfile(OUTPUTFILE,'fld',10,8*NCAR*NCAR)
      else
         NFLD=-1
      endif 
      if (IPPART.GT.0) then  !is output of particle distribution desired?
         IRECPAR=1
         NPAR =openbinfile(OUTPUTFILE,'par',11,8*NPART)
      else
         NPAR=-1
      endif

      if (IDMPFLD.ne.0) then   !dumped radiation field?
         NDUMP=openbinfile(OUTPUTFILE,'dfl',12,16*NCAR*NCAR)
      else
         NDUMP=-1
      endif
c       
      if (IDMPPAR.NE.0) then   !should the radiation field be dumped at the end?
         NDMP2=openbinfile(OUTPUTFILE,'dpa',13,8*NPART)
      else
         NDMP2=-1
      endif
c
      if (itdp.ne.0) CALL OPENTIMEREC(NCAR)          !prepare the CRTIME-record
c
c     open additional input files (MAGINFILE is opened in magfield.f)
c
      CALL READBEAMFILE(BEAMFILE)   !read external description file for beam 
c
      IF (INDEX(FIELDFILE,' ').NE.1) THEN   !does not start with a space
         NFIN=14
         CALL OPENFIELDFILE
      ELSE
         NFIN=-1
      ENDIF
      GOTO 120
 110  ierr1=PRINTERR(ERROPEN,FIELDFILE)
      NFIN=-1
c
 120  IF (INDEX(PARTFILE,' ').NE.1) THEN !does not start with a space
         NPIN=25
         CALL OPENPARTFILE
      ELSE
         NPIN=-1
      ENDIF
      GOTO 140
 130  ierr1=PRINTERR(ERROPEN,PARTFILE)
      NPIN=-1
c
c     distribution : open and initialize
c
 140  NDIS=READDISTFILE(DISTFILE,17) !open & initial external distribution file
c
c     check for boundary violation or unphysical input parameters
c
c
      IERR1=chk_bnd()           !check for boundary violation
      IF (ierr1.Lt.0) call last
c
      RETURN
c

      END  !of IOINIT
c
c
c
      FUNCTION READIN()
c     =================================================================
c     This routine reads in the user input files.
c     It assumes the standard fortran NAMELIST Format.
c     ------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
c
      INTEGER NIN
c
      NAMELIST /NEWRUN/
c
c     WIGGLER        
     +          AW0,XKX,XKY,WCOEFZ,XLAMD,FBESS0,DELAW,IERTYP,IWITYP,
     +          AWD,ISEED,
c     ELECTRON BEAM
     +          NPART,GAMMA0,DELGAM,RXBEAM,RYBEAM,ALPHAX,ALPHAY,
     +          EMITX,EMITY,XBEAM,YBEAM,PXBEAM,PYBEAM,CUTTAIL,CURPEAK,
c     RADIATION
     +          XLAMDS,PRAD0,ZRAYL,ZWAIST,
c     GRID-QUANTITIES   
     +          NCAR,LBC,RMAX0,DGRID,NSCR,NSCZ,NPTR,
c     CONTROL
     +          NWIG,DELZ,ZSEP,NSEC,IORB,ZSTOP,MAGIN,MAGOUT,NBINS,
     +          VERSION,
c     STRONG FOCUSING
     +          QUADF,QUADD,FL,DL,DRL,F1ST,QFDX,QFDY,SL,SOLEN,
c     LOADING 
     +          ILDGAM,ILDPSI,ILDX,ILDY,ILDPX,ILDPY,ITGAUS,LOUT,
c     OUTPUT
     +          IPHSTY,ISHSTY,IPPART,ISPART,IPRADI,ISRADI,
     +          IDUMP,IOTAIL,NHARM,IDMPPAR,IDMPFLD,ILOG,
c     EXTERNAL FILES
     +          BEAMFILE,FIELDFILE,MAGINFILE,MAGOUTFILE,OUTPUTFILE, 
     +          PARTFILE,DISTFILE,FILETYPE,  
c     TIME-DEPENDENCY
     +          CURLEN,NTAIL,NSLICE,SHOTNOISE,IALL,ITDP,IPSEED,  
c     SCAN
     +          ISCAN,NSCAN,SVAR,SCAN,  
c     EXTENSION
     +          ISRAVG,ISRSIG,ELOSS,CHARGE,NDCUT 
c     TEMPORARY INCLUDED PARAMETER

c
c     Initialize input/output
c
      NIN=8
      READIN=NOERR
      call preset
      WRITE(6,100)                                 !initialize input parameters
      Read(5,110) inputfile                        !Get input filename.
      Open(nin,file=inputfile,ERR=10,status='OLD') !Open input file.
      READ(NIN,newrun,ERR=20,END=20)                      !Read in namelist
      CLOSE(NIN)                                   !Close file 
      RETURN
c
 10   READIN=PRINTERR(ERROPENIN,inputfile)
      RETURN
 20   READIN=PRINTERR(ERRREADIN,inputfile)
      CALL LAST
      RETURN
c
c     format statements
c    
 100  Format('PLEASE ENTER INPUT FILE NAME ')
 110  Format(a30)
      END   !Readin
c
      SUBROUTINE READBEAMFILE(file)
c     =============================================================
c     read the file for external description of the electron beam
c     -------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'time.cmn'
      include 'sdds.cmn'
      INCLUDE "SDDSFortran.h"
c
      INTEGER NIN,i,ncol,ipar(15),ix,idata,j
      CHARACTER*30 file
      CHARACTER*50 cerr
      character*255 line
      real*8  ver,values(15)
c
      DIMENSION cNamesD(15)
      CHARACTER*30 cNamesD
      INTEGER SDDSNNINColumnIndex
      DIMENSION SDDSNNINColumnIndex(15)
c
      IF (INDEX(file,' ').EQ.1) RETURN

      IF (ISCAN.NE.0) THEN
          i=PRINTERR(ERRSCANEX,file)
          RETURN
      ENDIF    
c
c     Read file
c
      if (SDDS .EQ. 1) then
         cNamesD(1) = "s"
         cNamesD(2) = "gamma"
         cNamesD(3) = "dgamma"
         cNamesD(4) = "xemit"
         cNamesD(5) = "yemit"
         cNamesD(6) = "xrms"
         cNamesD(7) = "yrms"
         cNamesD(8) = "xavg"
         cNamesD(9) = "yavg"
         cNamesD(10) = "pxavg"
         cNamesD(11) = "pyavg"
         cNamesD(12) = "alphax"
         cNamesD(13) = "alphay"
         cNamesD(14) = "current"
         cNamesD(15) = "wakez"
         i = SDDS_InitializeInput_F(7,BEAMFILE)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         do j = 1,15
            i = SDDS_GetColumnIndex_F(7,cNamesD(j))
            if (i .EQ. -1) then
               call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
               stop 1
            endif
            SDDSNNINColumnIndex(j) = i
            i = SDDS_GetColumnType_F(7, SDDSNNINColumnIndex(j))
            if (i .NE. SDDS_DOUBLE) then
               write (6,*) "invalid column type in field file"
               stop 1
            endif
         enddo
         i = SDDS_ReadPage_F(7)
         if (i .EQ. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         NDATA = SDDS_RowCount_F(7)
         if (NDATA .GT. NSMAX) then
            Write (6,*)
            Write (6,*) '***** Too many records in beamfile'
            Write (6,*)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(1), TPOS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(2), TGAM0,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(3), TDGAM,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(4), TEMITX,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(5), TEMITY,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(6), TXRMS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(7), TYRMS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(8), TXPOS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(9), TYPOS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(10), TPXPOS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(11), TPYPOS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(12), TALPHX,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(13), TALPHY,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(14), TCURRENT,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_GetColumnData_F(7, SDDSNNINColumnIndex(15), TLOSS,
     +        NDATA, SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         i = SDDS_Terminate_F(7)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         endif
         return
      endif
      NIN=OPENTEXTFILE(FILE,'old',8)
      if (nin.lt.0) call last ! stop program on error
c
      NDATA=-1            !# of rows not defined
      idata=0             !# of rows read
      NCOL =15            !# of elements per line
      ver=0.1
      do i=1,ncol
         ipar(i)=i        !basic order of input
      enddo
c
 1    READ(NIN,100,ERR=20,end=50) line
c
c     processing line
c     
      call touppercase(line)
      call getfirstchar(line,ix)
c
      if (ix.eq.0) goto 1                     !empty line
      if (line(ix:ix).eq.'#') goto 1          !no comment used
      if (line(ix:ix).eq.'?') then
         call getbeamfileinfo(line,ipar,ncol,ndata,ver)  !check for add. info
         goto 1
      endif
c
      if ((ndata.lt.0).and.(ver.lt.1.0)) then !old version
         i=extractval(line,values,1)
         if (i.lt.0) then
            write(cerr,*) 'Line number of BEAMFILE cannot be determined'
            i=PRINTERR(ERRINPUT,cerr)
         endif
         ndata=nint(values(1))
         goto 1
      endif
c
      i=extractval(line,values,ncol)
      if (i.lt.0) then
        write(cerr,*) 'BEAMFILE data line ',idata+1,' has bad format'
        i=PRINTERR(ERRINPUT,cerr)
        CALL LAST
      endif
c
      idata=idata+1
c
c     set default values      
c
      TGAM0(idata)=GAMMA0
      TDGAM(idata)=DELGAM
      TEMITX(idata)=EMITX
      TEMITY(idata)=EMITY
      TXRMS(idata)=RXBEAM
      TYRMS(idata)=RYBEAM 
      TXPOS(idata)=XBEAM
      TYPOS(idata)=YBEAM
      TPXPOS(idata)=PXBEAM
      TPYPOS(idata)=PYBEAM
      TALPHX(idata)=ALPHAX
      TALPHY(idata)=ALPHAY
      TCURRENT(idata)=CURPEAK
      TLOSS(idata)=ELOSS
c
c     write over with input data
c
      DO j=1,NCOL
         if (ipar(j).eq.1 ) TPOS(idata)    =values(j)
         if (ipar(j).eq.2 ) TGAM0(idata)   =values(j)
         if (ipar(j).eq.3 ) TDGAM(idata)   =values(j)
         if (ipar(j).eq.4 ) TEMITX(idata)  =values(j)
         if (ipar(j).eq.5 ) TEMITY(idata)  =values(j)
         if (ipar(j).eq.6 ) TXRMS(idata)   =values(j)
         if (ipar(j).eq.7 ) TYRMS(idata)   =values(j)
         if (ipar(j).eq.8 ) TXPOS(idata)   =values(j)
         if (ipar(j).eq.9 ) TYPOS(idata)   =values(j)
         if (ipar(j).eq.10) TPXPOS(idata)  =values(j)
         if (ipar(j).eq.11) TPYPOS(idata)  =values(j)
         if (ipar(j).eq.12) TALPHX(idata)  =values(j)
         if (ipar(j).eq.13) TALPHY(idata)  =values(j)
         if (ipar(j).eq.14) TCURRENT(idata)=values(j)
         if (ipar(j).eq.15) TLOSS(idata)   =values(j)
      enddo 
c
      goto 1
c
 50   close(NIN)
c 
      if ((ndata.ge.0).and.(idata.ne.ndata)) then
         i=PRINTERR(ERRINWARN,'BEAMFILE has fewer lines than defined')
      endif
      ndata=idata
      if (idata.lt.2) then
       i=PRINTERR(ERRINPUT,'BEAMFILE contains less than 2 valid lines')
       call last
      endif
c
      return
c
 20   i=PRINTERR(ERRREAD,file)
      CLOSE(NIN)
      CALL LAST
      RETURN
c
c     format statement
c
 100  FORMAT(A)
c
      End     !READBEAMFILE
c 
      subroutine getbeamfileinfo(line,ipar,ncol,ndata,ver)
c     =================================================================
c     extract information from BEAMFILE
c     -----------------------------------------------------------------
c
      include 'genesis.def'
c
      character*(*) line
      character*255 cline
      integer ndata,ipar(*),i,n,ncol,ierr,j,ix1,ix2,haszpos,iarg
      real*8  ver,val
c
c     version number
c
      i=index(line,'VERSION') !check for version number
      n=len(line)
      if (i.gt.0) then
        ierr=extractnumber(line(i+7:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in BEAMFILE')
        else
           ver=val
        endif
        return
      endif 
c
c     line numbers
c
      i=index(line,'SIZE') !check for size argument (aka NDATA)
      if (i.gt.0) then
        ierr=extractnumber(line(i+4:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in BEAMFILE')
        else
           ndata=nint(val)
        endif
        return
      endif
c
c     colum order
c
      i=index(line,'COLUMNS') !check for colums headers
      if (i.gt.0) then
         do j=1,15
            ipar(j)=0
         enddo
         ncol=0
         cline=line(i+7:n)
         haszpos=0
 1       call getfirstchar(cline,ix1)
         if (ix1.gt.0) then
           ix2=255             !search backwards
           do j=255,ix1+1,-1   !for first space after ix1
             if (cline(j:j).eq.' ') ix2=j  
           enddo
           iarg=0
           if (index(cline(ix1:ix2),'ZPOS').ne.0) iarg=1
           if (index(cline(ix1:ix2),'GAMMA0').ne.0) iarg=2
           if (index(cline(ix1:ix2),'DELGAM').ne.0) iarg=3
           if (index(cline(ix1:ix2),'EMITX').ne.0) iarg=4
           if (index(cline(ix1:ix2),'EMITY').ne.0) iarg=5
           if (index(cline(ix1:ix2),'RXBEAM').ne.0) iarg=6
           if (index(cline(ix1:ix2),'RYBEAM').ne.0) iarg=7
           if (index(cline(ix1:ix2),'XBEAM').ne.0) iarg=8
           if (index(cline(ix1:ix2),'YBEAM').ne.0) iarg=9
           if (index(cline(ix1:ix2),'PXBEAM').ne.0) iarg=10
           if (index(cline(ix1:ix2),'PYBEAM').ne.0) iarg=11
           if (index(cline(ix1:ix2),'ALPHAX').ne.0) iarg=12
           if (index(cline(ix1:ix2),'ALPHAY').ne.0) iarg=13
           if (index(cline(ix1:ix2),'CURPEAK').ne.0) iarg=14
           if (index(cline(ix1:ix2),'ELOSS').ne.0) iarg=15
c
           if (iarg.eq.0) then
              do j=1,15
                 ipar(j)=j
              enddo   
              ncol=15
              j=PRINTERR(ERRINWARN,
     c              'Unrecognized information line in BEAMFILE')
              return
           else
              ncol=ncol+1 
              ipar(ncol)=iarg
              if (iarg.eq.1) haszpos=1
              cline=cline(ix2+1:255)
              if (ncol.lt.15) goto 1
           endif
         endif
         if (haszpos.lt.1) then
            do j=1,15
               ipar(j)=j
            enddo   
            ncol=15
            j=PRINTERR(ERRINWARN,
     c            'ZPOS column not specified in BEAMFILE')
            return
         endif
         return
      endif
      i=PRINTERR(ERRINWARN,'Unrecognized information line in BEAMFILE')
      return
      end
c
      subroutine readpart(ISLICE,xkper0)
c     =================================================================
c     load complete set of particle from file
c     -----------------------------------------------------------------
c
      include 'genesis.def'
      include 'input.cmn'
      include 'particle.cmn'
      include 'io.cmn' 
      include 'sdds.cmn'
      INCLUDE "SDDSFortran.h"
c
      integer j,i,islice
      real*8 xkper0
c
      INTEGER sddsslice,records,rows
      REAL*8 sddsz
c

      LOGICAL ISOP
c
      j=6*(ISLICE-1)+1 
      if (SDDS .EQ. 0) then
         READ(NPIN,REC=j,  ERR=100) (GAMMA(I),I=1,NPART)
         READ(NPIN,REC=j+1,ERR=100) (THETA(I),I=1,NPART)
         READ(NPIN,REC=j+2,ERR=100) (XPART(I),I=1,NPART)
         READ(NPIN,REC=j+3,ERR=100) (YPART(I),I=1,NPART)
         READ(NPIN,REC=j+4,ERR=100) (PX(I),I=1,NPART)
         READ(NPIN,REC=j+5,ERR=100) (PY(I),I=1,NPART)
      else
 1       i = SDDS_ReadPage_F(6)
         if (i .EQ. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         SDDSNPINCurrentPage = SDDSNPINCurrentPage + 1
         i = SDDS_GetParameterData_F(6, SDDSNPINSliceIndex, sddsslice,
     +        SDDS_LONG)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetParameterData_F(6, SDDSNPINZIndex, sddsz,
     +        SDDS_DOUBLE)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         if (SDDSNPINPRIZIndex .EQ. -1) then
            records = 1
         else
            i = SDDS_GetParameterData_F(6, SDDSNPINPRIZIndex, records,
     +           SDDS_LONG)
            if (i .NE. 1) then
               call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
               stop 1
            end if
         end if
         if (sddsslice .NE. ISLICE) then
            write (6,*) 'unable to find need Z value in particle file'
            stop 1
         end if
         if (ISLICE * records .NE. SDDSNPINCurrentPage) then
            goto 1
         end if
         i = SDDS_GetParameterData_F(6, SDDSNPINCurrentIndex, XCUREN,
     +        SDDS_DOUBLE)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         rows = SDDS_RowCount_F(6)
         i = SDDS_GetColumnData_F(6,SDDSNPINColumnIndex(1),GAMMA,NPMAX,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetColumnData_F(6,SDDSNPINColumnIndex(2),THETA,NPMAX,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetColumnData_F(6,SDDSNPINColumnIndex(3),XPART,NPMAX,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if

         i = SDDS_GetColumnData_F(6,SDDSNPINColumnIndex(4),YPART,NPMAX,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetColumnData_F(6,SDDSNPINColumnIndex(5),PX,NPMAX,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetColumnData_F(6,SDDSNPINColumnIndex(6),PY,NPMAX,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
      endif
c
c     calculate init. perpendicular velocity (needed in first call of TRACK)
c
      DO I=1,NPART      
        xpart(i)=xpart(i)*xkper0
        ypart(i)=ypart(i)*xkper0
	BTPAR(I)=DSQRT(1.d0-(PX(I)**2+PY(I)**2+1.)/GAMMA(I)**2)     !parallel velocity
      ENDDO
c
      return
 100  i=PRINTERR(ERRREAD,'PARTFILE')
      CALL LAST
      RETURN
      end
c

      subroutine readslice(nget,x,px,y,py,g,tmin,tmax)
c     =================================================================
c     read slice [tmin,tmax] of particle from distribution file
c     -----------------------------------------------------------------
c
      include 'genesis.def'
      include 'io.cmn'

      character*255 line,cerr
      integer nget,n0,i,ndata,ix
      real*8 x(*),px(*),y(*),py(*),g(*)
      real*8 tx,tpx,ty,tpy,tt,tg
      real*8 tmin,tmax,values(10)
c

      n0=-1
      nget=0
      ndata=0
c
 1    READ(ndis,200,ERR=100,END=50) line
c
c     processing line 
c      
      call touppercase(line)
      call getfirstchar(line,ix)
      if (ix.eq.0) goto 1             !empty line
      if (line(ix:ix).eq.'#') goto 1  !comment line
      if (line(ix:ix).eq.'?') goto 1  !information allready processed
c
      if ((n0.lt.0).and.(distversion.lt.1.0)) then
         i=extractval(line,values,1)
         if (i.lt.0) then
            write(cerr,*) 'DISTFILE has invalid input line: ',line
            i=PRINTERR(ERRINPUT,cerr)
            CALL LAST
         endif
         n0=nint(values(1))
         goto 1
      endif
c
c     get record-tuple 
c
      i=extractval(line,values,ncoldis)
      if (i.lt.0) then
          write(cerr,*) 'DISTFILE has invalid input line: ',line
          i=PRINTERR(ERRINPUT,cerr)
          CALL LAST
      endif
      ndata=ndata+1
      do i=1,ncoldis
         if (icolpar(i).eq.1) tx =values(i)
         if (icolpar(i).eq.2) tpx=values(i)
         if (icolpar(i).eq.3) ty =values(i)
         if (icolpar(i).eq.4) tpy=values(i)
         if (icolpar(i).eq.5) tt =values(i)
         if (icolpar(i).eq.6) tg =values(i)
      enddo
      if (iconv2t.ne.0 ) tt=-tt/3.e8           !convert from z to t
      if (iconv2g.ne.0 ) tg=dsqrt(tg*tg+1.d0)  !convert from p to gamma
      if (iconv2px.ne.0) tpx=tpx*tg            !convert from x' to px
      if (iconv2py.ne.0) tpy=tpy*tg            !convert from y' to py
c
      if ((tt.ge.tmin).and.(tt.le.tmax)) then !in slice
           nget=nget+1
           x(nget) =tx                        !add to raw distribution
           px(nget)=tpx
           y(nget) =ty
           py(nget)=tpy
           g(nget) =tg
      endif   
      goto 1
c      
 50   rewind(ndis)              !set file pointer back
      return 
c
 100  i=PRINTERR(ERRREAD,'DISTFILE')
      CALL LAST
      RETURN
c
 200  FORMAT(A)
c
      end
c
      Function READDISTFILE(file,nio)
c     ==================================================================
c     open an external file containing the distribution. Read the file 
c     geting the parameter range etc.
c     ------------------------------------------------------------------
c
      include 'genesis.def'
      include 'particle.cmn'
      include 'input.cmn'
      include 'io.cmn'
c
      character*(*) file
      character*255 cerr,line
      integer   nio,nget,i,ix
      real*8 tt,values(10)
c      
      ndistsize=-1
      nget=0
c
c     default settings
c
      iconv2g=1       !needs to convert from p to gamma
      iconv2t=0       !long coordinate is time
      iconv2px=0      !convert from x' to px
      iconv2py=0      !convert from y' to py
      distversion=0.1 !first number is number of particles
      ncoldis=6       !6 dimension
      do i=1,10
         icolpar(i)=i ! x,px,y,py,t,gamma,id order
      enddo
c      
      READDISTFILE=-1
      if (INDEX(file,' ').eq.1) return   !no file selected
c
      nio=opentextfile(file,'old',nio)
      READDISTFILE=nio
      if (nio.lt.0) call last
c
 1    read(nio,200,ERR=100,END=50) line
c
c     processing line 
c      
      call touppercase(line)
      call getfirstchar(line,ix)
      if (ix.eq.0) goto 1             !empty line
      if (line(ix:ix).eq.'#') goto 1  !comment line
      if (line(ix:ix).eq.'?') then    !read information line
         call getdistfileinfo(line)
         goto 1
      endif
c
      if ((ndistsize.lt.0).and.(distversion.lt.1.0)) then
         i=extractval(line,values,1)
         if (i.lt.0) then
            cerr='DISTFILE has invalid input line:'//line
            i=PRINTERR(ERRINPUT,cerr)
            CALL LAST
         endif
         ndistsize=nint(values(1))
         goto 1
      endif
c
c     get record-tuple 
c
      i=extractval(line,values,ncoldis)
      if (i.lt.0) then
          cerr='DISTFILE has invalid input line:'//line
          i=PRINTERR(ERRINPUT,cerr)
          CALL LAST
      endif
c
      nget=nget+1
      do i=1,ncoldis
         if (icolpar(i).eq.5) tt=values(i)    !catch time value
      enddo
      if (iconv2t.ne.0) tt=-tt/3.e8           !convert from z to t
c
      if (nget.eq.1) then
         tdmin=tt
         tdmax=tt
      endif
      if (tt.lt.tdmin) tdmin=tt              !adjust min and max
      if (tt.gt.tdmax) tdmax=tt
      goto 1
c
 50   rewind(nio)               !go back to file beginning
c  
      if ((ndistsize.ge.0).and.(nget.ne.ndistsize)) then
         i=PRINTERR(ERRINWARN,'DISTFILE has fewer lines than defined')
      endif
c
      if (NDCUT.le.0) NDCUT=nget*nharm/npart    !self optimizing
      if (NDCUT.le.0) NDCUT=1
c
      if (charge.le.0) then
         i=PRINTERR(ERRINPUT,'CHARGE for DISTFILE is not defined')
         CALL LAST
      endif
c
      delcharge=charge/dble(nget)
      dtd=(tdmax-tdmin)/dble(ndcut)
c
      RETURN
c
 100  READDISTFILE=PRINTERR(ERRREAD,file)
      CALL LAST
      RETURN
c
 200  FORMAT(A)
c
      END
c

      subroutine getdistfileinfo(line)
c     =================================================================
c     extract information from BEAMFILE
c     -----------------------------------------------------------------
c
      include 'genesis.def'
      include 'input.cmn'
      include 'io.cmn'
c
      character*(*) line
      character*255 cline
      integer n,i,idx,ix1,ix2,ncount,narg,iarg,j,ierr
      real*8  val
c
      n=len(line)
      call getfirstchar(line,idx) ! get first character should be identical to INDEX !!
c
c     version number
c
      i=index(line,'VERSION') !check for version number
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+7:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in DISTFILE')
        else
           distversion=val
        endif
        return
      endif 
c
c     beam charge
c
      i=index(line,'CHARGE')
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+6:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in DISTFILE')
        else
           charge=val
        endif
        return
      endif 
c
c     cuts in long. phase space
c
      i=index(line,'NDCUT') 
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+5:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in DISTFILE')
        else
           NDCUT=nint(val)
        endif
        return
      endif 
c
c     size of record
c
      i=index(line,'SIZE') 
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+4:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in DISTFILE')
        else
           ndistsize=nint(val)
        endif
        return
      endif 
c
c     Get order and dimension
c

      i=index(line,'COLUMNS') !check for colums headers
      if (i.gt.0) then
         do j=1,10
            icolpar(j)=0
         enddo
         ncoldis=0
         ncount=0
         cline=line(i+7:len(line))
 1       call getfirstchar(cline,ix1)
         if (ix1.gt.0) then
           ix2=255             !search backwards
           do j=255,ix1+1,-1   !for first space after ix1
             if (cline(j:j).eq.' ') ix2=j  
           enddo
           iarg=0
           if (index(cline(ix1:ix2),'X').ne.0) iarg=1
           if (index(cline(ix1:ix2),'PX').ne.0) iarg=2
           if (index(cline(ix1:ix2),'XPRIME').ne.0) iarg=-2
           if (index(cline(ix1:ix2),'Y').ne.0) iarg=3
           if (index(cline(ix1:ix2),'PY').ne.0) iarg=4
           if (index(cline(ix1:ix2),'YPRIME').ne.0) iarg=-4
           if (index(cline(ix1:ix2),'T').ne.0) iarg=5
           if (index(cline(ix1:ix2),'Z').ne.0) iarg=-5
           if (index(cline(ix1:ix2),'P').ne.0) iarg=6
           if (index(cline(ix1:ix2),'GAMMA').ne.0) iarg=-6
c
           ncount=ncount+1
           narg=narg+abs(iarg)
           ncoldis=ncoldis+1
           icolpar(ncoldis)=abs(iarg)
           if (iarg.eq.2)  iconv2px=0
           if (iarg.eq.-2) iconv2px=1
           if (iarg.eq.4)  iconv2py=0
           if (iarg.eq.-4) iconv2py=1
           if (iarg.eq.5)  iconv2t=0
           if (iarg.eq.-5) iconv2t=1
           if (iarg.eq.6)  iconv2g=1
           if (iarg.eq.-6) iconv2g=0
           cline=cline(ix2+1:255)
           if (ncount.lt.10) goto 1
         endif

         if (narg.ne.21) then
            j=PRINTERR(ERRINWARN,
     c            'Not all dimensions defined in DISTFILE')
            call last
         endif
         return
      endif
c
c     unrecognized
c
      i=PRINTERR(ERRINWARN,'Unrecognized information line in BEAMFILE')
      return
      end
c
      Subroutine PRESET
c     ==================================================================
c     Sets default values of program inputs.
c     Default parameter modelled after the PEGASUS FEL      
c     ------------------------------------------------------------------
c
      include 'genesis.def'
      INCLUDE 'input.cmn'
c
      INTEGER I
c
c     WIGGLER PARAMETERS:
c
      AW0    = 0.735          !Dimensionless wiggler amplitude (RMS).
      XKX       = 0.0D0       !Weak focusing strength in x-plane.
      XKY       = 1.0D0       !Weak focusing strength in x-plane.
      WCOEFZ(1) = 0.0D0       !FIELD Taper start location in z.
      WCOEFZ(2) = 0.0D0       !FIELD Taper gradient
      WCOEFZ(3) = 0.0D0       !FIELD Taper model
      DELAW  = 0.0D0          !relative spread in AW
      IERTYP = 0              !field error distribution type (<0 = correlated)
      IWITYP = 0              !wiggler type (0=planar, 1= wiggler)
      AWD    = 0.735          !virtual wiggler field for drift space
      ISEED  = -1             !Initial seed for wiggler error generation
      FBESS0 = 0.0d0          !Beam-Radiation Coupling
      XLAMD  = 2.05D-2        !Wiggler period  (m)
c
c     ELECTRON BEAM PARAMETERS:
c
      NPART  = 8192           !Number of simulation particles.
      GAMMA0 = 35.2D0         !Electron beam lorentz factor (energy)
      DELGAM = 5.d-3          !RMS Energy (gamma) spread 
      RXBEAM = 112.1D-6       !RMS Beam size in x-plane 
      RYBEAM = 112.1D-6       !RMS Beam size in y-plane 
      ALPHAX= 0D0             !Twiss alpha parameter in x
      ALPHAY= 0D0             !Twiss alpha parameter in y
      EMITX  = 2.0D-6         !Normalized emittance x-plane (pi m-rad)
      EMITY  = 2.0D-6         !Normalized emittance y-plane (pi m-rad)
      XBEAM  = 0.d0           !center in x (m)
      YBEAM  = 0.d0           !center in y (m)
      PXBEAM  = 0.d0          !center in px 
      PYBEAM  = 0.d0          !center in py
      CUTTAIL = -1.           !no collimation transverse tail
      CURPEAK=2.5D2           !peak current
c
c     RADIATION:
c
      XLAMDS = 12.852D-6      !Output radiation wavelength
      PRAD0  = 1.00D01        !Input power
      ZRAYL  = .500D0         !Rayleigh range (of radiation)
      ZWAIST = 0.0D0          !Z location of (radiation) waist.
c
c     NUMERICAL CONTROL PARAMETERS:
c
      ILDGAM = 5              !Energy loading parameter.
      ILDPSI = 7              !Phase loading parameter.
      ILDX   = 1              !x-plane loading parameter.
      ILDY   = 2              !y-plane loading parameter.
      ILDPX  = 3              !x momentum loading parameter.
      ILDPY  = 4              !y momentum loading parameter.
      ITGAUS = 1              !Gaussian (<>0) or Uniform (=0) loading
      NBINS  = 4              !# of bins in the phase coordinate
c
c     MESH DISCRETIZATION:
c
      NCAR   = 151            !mesh points in one dimension (xy-grid).
      LBC    = 0              !boundary Condition (0=Diriqlet,<>0 Neumann)
      RMAX0  = 9.d0           !Maximum edge of grid.
      NSCR   = 1              !# radial modes for space charge
      NSCZ   = 1              !# longitudinal modes for space charge 
      NPTR   = 40             !Radial grid points for space charge
      DGRID  = 0.d0           !grid size(-DGRID to DGRID) if DGRID > 0
c
c     INTEGRATION CONTROL PARAMETER:
c
      NWIG =  98              !Number of undulator periods per module
      ZSEP =  1.              !Seperation of slices in units of XLAMDS
      DELZ =  1.0             !integration step in units of XLAMD
      NSEC = 1                !number of sections    
      IORB = 0                !Orbit correction flag (<>0 -> use term).
      ZSTOP=-1.               !stop simulation @ z=zstop 
      MAGIN=0                 !read magnetic field description if MAGIN <>0
      MAGOUT=0                !write magnetic field description if MAGOUT <>0
      VERSION=0.1             !assume oldest version
c
c     SCAN CONTROL PARAMETER
c
      ISCAN=0                 !>0 -> ISCANth parameter selected
      NSCAN=3                 !Number of scans  
      SVAR=0.01               !rel. Variation [-SVAL,SVAL]                
      SCAN=' '                !scan defined by name
c
c     OUTPUT CONTROL PARAMETERS:
c
      IPHSTY = 1              !History - steps in z 
      IPRADI = 0              !radiation field - steps in z
      IPPART = 0              !particles - steps in z
      ISHSTY = 1              !History - steps in t
      ISRADI = 1              !radiation field - steps in t
      ISPART = 1              !particles - steps in t
      DO I=1,11
         LOUT(I)=1            !Flags for output
      ENDDO
      LOUT(6)=0               !disable diffraction calculation
      DO i=12,14+NHMAX
         LOUT(i)=0 
      ENDDO
      IOTAIL=0                !<>0 => output include also slippage
      IDUMP=0                 !<>0 => dump complete radiation field.
      NHARM=1                 !# or harmonics in the bunching factor
      IDMPFLD=0               !<>0 => dump radiation field
      IDMPPAR=0               !<>0 => dump particle distribution
      ILOG=0                  !<>0 => terminal output written to file
c
c     FILE NAMES
c   
      BEAMFILE=' '            !beam description fiel
      FIELDFILE=' '           !input radiation field
      MAGINFILE=' '           !input magnetic field
      MAGOUTFILE=' '          !output magnetic field
      OUTPUTFILE=' '          !output file
      PARTFILE=' '            !input particle file
      DISTFILE=' '            !input distribution file
      FILETYPE='ORIGINAL'     !filetype for output files (SDDS,XML)
c
c     FOCUSSING:
c
      QUADF = 1.23D0          !Quad focus strength
      QUADD = 0.00D0          !Quad defocus strength
      QFDX  = 0.0             !Quadrupole offset in x
      QFDY  = 0.0             !Quadrupole offset in y
      FL    = 98.             !Focus section length
      DL    = 0.              !Defocus section length
      DRL   = 0.              !Drift length
      F1ST  = 0.              !start FODO at this point 
      SOLEN = 0.              !strength of solenoid field
      SL    = 0.              !Solenoid length
c
c     TIME DEPENDENCY PARAMETERS:
c
      CURLEN=1.d-3            !rms bunch length in m (<0 -> uniform beam)
      NTAIL=-253              !starting slice in beam relative to current peak
      NSLICE=408              !#slices
      ITDP=0                  !<>0 => time dependent code
      SHOTNOISE=1.d0          !scaling of the shotnoise meanvalue
      IALL=0                  !<>0 => load phasespace with same loading
      IPSEED=-1               !seeding of the random numbers for shotnoise
      CHARGE=-1.d0            !charge if external distirbution is used
      NDCUT=-1                !=<0 self optimized binning of ext. dist.
c
c     EXTENSIONS
c
      ISRAVG=0                !<>0 enables energy loss by incorerent radiation
      ISRSIG=0                !<>0 enables growth of energy spread by Inc. SR.
      ELOSS=0.0               !energy loss per meter
c
      Return
      End     !PRESET
c
c
c     
c
      SUBROUTINE OPENPARTFILE
c     ==================================================================
c     Opens the PARTFILE
c     ------------------------------------------------------------------

      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'io.cmn'
      include 'sdds.cmn'
      INCLUDE "SDDSFortran.h"
c
      INTEGER i,j,ierr1
      DIMENSION cNamesD(6)
      CHARACTER*6 cNamesD
c
      if (SDDS .EQ. 0) then
         OPEN(NPIN,FILE=PARTFILE,ACCESS='DIRECT',status='old',
     +        RECL=NPART*8,ERR=130)
      else
         cNamesD(1) = "Energy"
         cNamesD(2) = "Theta"
         cNamesD(3) = "XPosition"
         cNamesD(4) = "YPosition"
         cNamesD(5) = "XMomentum"
         cNamesD(6) = "YMomentum"
c
         i = SDDS_InitializeInput_F(6,PARTFILE)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetParameterIndex_F(6,'Slice')
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         SDDSNPINSliceIndex = i
         i = SDDS_GetParameterType_F(6, SDDSNPINSliceIndex) 
         if (i .NE. SDDS_LONG) then
            write (6,*) "invalid parameter type in particle file"
            stop 1
         end if
         i = SDDS_GetParameterIndex_F(6,'Current')
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         SDDSNPINCurrentIndex = i
         i = SDDS_GetParameterType_F(6, SDDSNPINCurrentIndex) 
         if (i .NE. SDDS_DOUBLE) then
            write (6,*) "invalid parameter type in particle file"
            stop 1
         end if
         i = SDDS_GetParameterIndex_F(6,'Z')
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         SDDSNPINZIndex = i
         i = SDDS_GetParameterType_F(6, SDDSNPINZIndex) 
         if (i .NE. SDDS_DOUBLE) then
            write (6,*) "invalid parameter type in particle file"
            stop 1
         end if
         i = SDDS_GetParameterIndex_F(6,'ParticleRecordsInZ')
         if (i .EQ. -1) then
            SDDSNPINPRIZIndex = -1
         else
            SDDSNPINPRIZIndex = i
            i = SDDS_GetParameterType_F(6, SDDSNPINPRIZIndex) 
            if (i .NE. SDDS_LONG) then
               write (6,*) "invalid parameter type in particle file"
               stop 1
            end if
         end if
         do j = 1,6
            i = SDDS_GetColumnIndex_F(6,cNamesD(j))
            if (i .EQ. -1) then
               call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
               stop 1
            end if
            SDDSNPINColumnIndex(j) = i
            i = SDDS_GetColumnType_F(6, SDDSNPINColumnIndex(j)) 
            if (i .NE. SDDS_DOUBLE) then
               write (6,*) "invalid column type in particle file"
               stop 1
            end if
         end do
      endif

      goto 140
 130  ierr1 = PRINTERR(ERROPEN,PARTFILE)
      NPIN=-1
 140  Return
      End

      SUBROUTINE OPENFIELDFILE
c     ==================================================================
c     Opens the PARTFILE
c     ------------------------------------------------------------------

      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'io.cmn'
      include 'sdds.cmn'
      INCLUDE "SDDSFortran.h"
c
      INTEGER i,ierr1
c
      if (SDDS .EQ. 0) then
         OPEN(NFIN,FILE=FIELDFILE,ACCESS='DIRECT',status='old',
     +        RECL=NCAR*NCAR*16,ERR=110) 
      else
         i = SDDS_InitializeInput_F(1,FIELDFILE)
         if (i .NE. 1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetColumnIndex_F(1,'ReA')
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         SDDSNFINRealIndex = i
         i = SDDS_GetColumnIndex_F(1,'ImA')
         if (i .EQ. -1) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         SDDSNFINImagIndex = i
         
         SDDSNFINCurrentPage = 0
         i = SDDS_GetColumnType_F(1, SDDSNFINRealIndex) 
         if (i .NE. SDDS_DOUBLE) then
            write (6,*) "invalid column type in field file"
            stop 1
         end if
         i = SDDS_GetColumnType_F(1, SDDSNFINImagIndex) 
         if (i .NE. SDDS_DOUBLE) then
            write (6,*) "invalid column type in field file"
            stop 1
         end if
      endif

      goto 120
 110  ierr1=PRINTERR(ERROPEN,FIELDFILE)
      NFIN=-1
 120  Return
      End
