      Subroutine LOADBEAM(islice,XKPER0)
c     ===================================================================
c     This routine fills the phase space for one slice of phase space
c     ------------------------------------------------------------------
c
      Include 'genesis.def'
      INCLUDE 'input.cmn'
      Include 'particle.cmn'
      include 'io.cmn'
c
      REAL*8 enum,snoise,sn1,sn2,ratio,xkper0
      INTEGER I,IP,MPART,islice
c
c     initialize particle loss
c
      lost=0
      DO i=1,NPART
         LOSTID(i)=0
      ENDDO
c
      IF (IALL.NE.0) THEN
        ILDPSI=-ABS(ILDPSI)   ! reinitialize all hammersley sequences
      END if         
c
c     Fill Phase
c
      MPART=NPART/NBINS               !Particles per bin
c
      Do IP=1,MPART
        THETA(IP)=hammv(ILDPSI)*2.*pi/DBLE(NBINS)-pi  !Load in first bin
      END DO 
c
c     branches for different loading methods
c
      if (npin.gt.0) then
         call readpart(ISLICE,xkper0)  !if reading from file
         return                        !skip the rest (loading is done)
      endif 
      if (ndis.gt.0) then
         CALL LOADDIST(MPART,ISLICE)   !load from distribution file
      else   
         CALL LOADQUIET(MPART)         !internal load (quiet start)
      endif   
c
c     normalized transverse position
c 
     
      Do IP=1,MPART
         XPART(IP)=XPART(IP)*XKPER0
         YPART(IP)=YPART(IP)*XKPER0
      END DO 
c
c     Mirror particle in remaining bins
c         
      DO I=1,NBINS-1
       DO IP=1,MPART
        XPART(IP+I*MPART)=XPART(IP)
        YPART(IP+I*MPART)=YPART(IP)
        PX(IP+I*MPART)=PX(IP)
        PY(IP+I*MPART)=PY(IP)
        GAMMA(IP+I*MPART)=GAMMA(IP)
        THETA(IP+I*MPART)=THETA(IP)+2.*DBLE(i)*pi/DBLE(NBINS)
        LOSTID(IP+I*MPART)=LOSTID(IP)
       End DO 
      End Do   
c
c     Add Shotnoise 
c
      ENUM=XCUREN*XLAMDS/ce                           !#electron in slice
      if (ENUM.LT.10) ENUM = 10            !catch low current error
      ratio=float(NPART)/ENUM/zsep
      snoise=SQRT(3.d0*float(NPART)/ENUM/zsep) !shot noise parameter     
c
c     For NPART ~ ENUM the approximation is not precise enough
c      
      if (ratio.gt.0.02) then
         sn1=snoise
         sn2=pi*0.5
 1       snoise=(sn1+sn2)*0.5
         ratio=sin(snoise)**2/snoise/snoise-1.+float(NPART)/ENUM/ZSEP 
         if (ratio.lt.0) then
            sn2=snoise
         else
            sn1=snoise
         endif
         if ((sn2-sn1).GT.0.002) goto 1 
      endif
c
      snoise=snoise*SHOTNOISE   
c
      if (snoise.Gt.TINY) then
        DO IP=1,NPART
          THETA(IP)=THETA(IP)+SNOISE*(1.D0-2.D0*RAN1(IPSEED))
        END DO
      endif  
c
c     calculate init. perpendicular velocity (needed in first call of TRACK)
c
      DO IP=1,NPART      
	BTPAR(IP)=DSQRT(1.d0-(PX(IP)**2+PY(IP)**2+1.)/GAMMA(IP)**2)     !parallel velocity
      ENDDO
c
      Return
      End     !loadbeam
c
c
      subroutine loadquiet(MPART)
c     ==================================================================
c     do quiet loading of transverse phase space
c     -------------------------------------------------------------------
c
      include 'genesis.def'
      include 'particle.cmn'
      include 'input.cmn'
c
      INTEGER MPART,IP
      REAL*8  xy,x,xx,y,yy,fac,xd,yd,pxd,pyd,xpd,ypd
      REAL*8  betaxinv,betayinv,r2p,r2
c
c     Fill energy with standard diviation 1
c
      Do IP=1,MPART
         GAMMA(IP)=gasham(ILDGAM)
      End Do
c
      MPART=MPART/2     !only half particles per one bin, others symmetric!
c
c     Fill x,y,px,py parabolic between [-1,1]
c
      IP=0
 10   Continue
      IP=IP+1
 20   Continue
      XPART(IP) = hammv(ILDX )*2.0D0-1.0d0
      YPART(IP) = hammv(ILDY )*2.0D0-1.0d0
      PX(IP)    = hammv(ILDPX)*2.0D0-1.0d0
      PY(IP)    = hammv(ILDPY)*2.0D0-1.0d0
      R2 = XPART(IP)**2+ YPART(IP)**2
      If(R2.GE.1.)goto 20
      R2P=PX(IP)**2+PY(IP)**2+R2
      If(R2P.GE.1.)goto 20
      If(IP.LT.MPART)goto 10
c
c     enforce profile
c
       Do IP=1,MPART                              
         R2=XPART(IP)**2+YPART(IP)**2+PX(IP)**2+PY(IP)**2
         If(r2.gt.tiny) then
            fac=1.0d0                                        !parabolic as default
            If(ITGAUS.eq.1)fac=dsqrt(-1.d0*dlog(1.d0-r2)/r2) !gaussian
            If(ITGAUS.eq.2)fac=1.d0/dsqrt(r2)                !step
            XPART(IP)=XPART(IP)*fac
            YPART(IP)=YPART(IP)*fac
            PX(IP)=PX(IP)*fac
            PY(IP)=PY(IP)*fac
         End If
      End Do
c
c     Correct phasespace loading 
c
      Do IP=1,MPART                         !<x>=<y>=0 locally
         XPART(IP+MPART)=-XPART(IP)
         YPART(IP+MPART)=-YPART(IP)
         PX(IP+MPART)=-PX(IP)
         PY(IP+MPART)=-PY(IP)
      End Do
c
      MPART=MPART*2
c
c     normalize distritution -> rms value is 1.
c
      call compmom(mpart,x,xx,y,yy,xy,xpart,ypart)  !compute moments
      xd=1./DSQRT(xx)
      yd=1./DSQRT(yy)
      DO IP=1,MPART
         XPART(IP)=XPART(IP)*xd
         YPART(IP)=YPART(IP)*yd
      ENDDO
c
      call compmom(mpart,x,xx,y,yy,xy,px,py)  !compute moments
      xd=1./DSQRT(xx)
      yd=1./DSQRT(yy)
      DO IP=1,MPART
         PX(IP)=PX(IP)*xd
         PY(IP)=PY(IP)*yd
      ENDDO
c
      CALL CUT_TAIL(MPART)
c
      call compmom(mpart,x,xx,y,yy,xy,GAMMA,THETA)  !compute moments
      xx=xx-x*x
      xd=1./DSQRT(xx)
      DO IP=1,MPART
         GAMMA(IP)=(GAMMA(IP)-x)*xd
      ENDDO
c
c     sclae and shift distribution
c
      betaxinv=emitx/RXBEAM**2                !calculation of 1/betax => !SR
      betayinv=emity/RYBEAM**2                !no sigularity for emit=0  !SR
      pxd=rxbeam*betaxinv                         !rms px at x=0
      pyd=rybeam*betayinv                         !rms py at y=0
      xpd=-alphax*betaxinv                    !slope of phase space ellipse
      ypd=-alphay*betayinv
      Do IP=1,MPART
         px(IP)=xpd*rxbeam*xpart(IP)+px(IP)*pxd   !scale+shift px
	 py(IP)=ypd*rybeam*ypart(IP)+py(IP)*pyd   !scale+shift py
	 xpart(IP)=xpart(ip)*rxbeam               !scale x to right size
	 ypart(IP)=ypart(ip)*rybeam               !scale y to right size
         gamma(IP)=gamma0+delgam*gamma(ip)
      End Do
c
c     Normalize and mirror phase space
c
      Do IP=1,MPART
         XPART(IP)=(XPART(IP)+XBEAM)
         YPART(IP)=(YPART(IP)+YBEAM)
         PX(IP)=(PX(IP)+PXBEAM)
         PY(IP)=(PY(IP)+PYBEAM)
      END DO
 
      RETURN
      END
c
c
      subroutine loaddist(MPART,ISLICE)
c     =================================================================
c     load slice from distribution
c     -----------------------------------------------------------------
c
      include 'genesis.def'
      include 'input.cmn'
      include 'particle.cmn'
c
      integer mpart,mget,n1,n2,i,ISLICE
      real*8 t0,t1,xa,xs,ya,ys,pxa,pxs,pya,pys,ga,gs
      real*8 tmp1,tmp2,tmp3
c
      t0=tdmax-(NTAIL+ISLICE-1)*ZSEP*XLAMDS/3.d8-0.5*dtd !note sign for t -> z conversion
      t1=t0+dtd
      CALL READSLICE(MGET,XPART,PX,YPART,PY,GAMMA,t0,t1)
      XCUREN=delcharge*dble(mget)/dtd    
c
      if (mget.lt.5) then
         do i=mget+1,mpart
            xpart(i)=0.d0
            ypart(i)=0.
            px(i)=0.
            py(i)=0.
            gamma(i)=gamma0
         enddo
         return
      endif   
c      
c     adjust and normalize distirbution    
c 
      call compmom(mget,xa,xs,pxa,pxs,tmp3,xpart,px)
      call compmom(mget,ya,ys,pya,pys,tmp3,ypart,py)
      call compmom(mget,ga,gs,tmp1,tmp2,tmp3,gamma,xpart)
      xs =dsqrt(xs-xa*xa)
      pxs=dsqrt(pxs-pxa*pxa)
      ys =dsqrt(ys-ya*ya)
      pys=dsqrt(pys-pya*pya)
      gs =dsqrt(gs-ga*ga)
      if (xs .lt.TINY) xs =1.d0
      if (pxs.lt.TINY) pxs=1.d0
      if (ys .lt.TINY) ys =1.d0
      if (pys.lt.TINY) pys=1.d0
      if (gs .lt.TINY) gs =1.d0
      call scaledist(xpart,mget,-xa ,1.d0/xs )
      call scaledist(px   ,mget,-pxa,1.d0/pxs)
      call scaledist(ypart,mget,-ya ,1.d0/ys )
      call scaledist(py   ,mget,-pya,1.d0/pys)
      call scaledist(gamma,mget,-ga,1.d0/gs)
c
c     massage proto-distribution
c
      if (mget.ge.mpart) then
c
c         remove particles         
c 
         do i=mpart,mget-1
            n1=int(mget*ran1(iseed))+1
            call switch(xpart,px,ypart,py,gamma,n1,mget)
            mget=mget-1
         enddo   
      else
c
c        add particle
c
   1     n1=int(dble(mget)*ran1(iseed))+1
         call neighbor(xpart,px,ypart,py,gamma,n1,n2,mget,iseed)
         xpart(mget+1) =xpart(n1) +(xpart(n2) -xpart(n1) )
     +              *(0.5*(ran1(iseed)+ran1(iseed)))
         px(mget+1)=px(n1)+(px(n2)-px(n1))
     +              *(0.5*(ran1(iseed)+ran1(iseed)))
         ypart(mget+1) =ypart(n1) +(ypart(n2) -ypart(n1) )
     +              *(0.5*(ran1(iseed)+ran1(iseed)))
         py(mget+1)=py(n1)+(py(n2)-py(n1))
     +              *(0.5*(ran1(iseed)+ran1(iseed)))
         gamma(mget+1) =gamma(n1)+(gamma(n2) -gamma(n1))
     +              *(0.5*(ran1(iseed)+ran1(iseed)))
         mget=mget+1
         if (mget.lt.mpart) goto 1
      endif
c
c     scale back
c
      call scaledist(xpart,mpart,xa/xs,xs)
      call scaledist(px   ,mpart,pxa/pxs,pxs)
      call scaledist(ypart,mpart,ya/ys,ys)
      call scaledist(py   ,mpart,pya/pys,pys)
      call scaledist(gamma,mpart,ga/gs,gs)
c
      return
      end
c
c
      Subroutine compmom(n,x,x2,y,y2,xy,xa,ya)
c     ==================================================================
c     compute moments
c     ------------------------------------------------------------------
c
      INTEGER n,i
      REAL*8 xa,ya,x,x2,y,y2,xy,dd
      Dimension xa(*),ya(*)                   !>=n
c
      x =0.0d0
      x2=0.0d0
      y =0.0d0
      y2=0.0d0
      xy=0.0d0
      Do i=1,n
         x =x +xa(i)
         y =y +ya(i)
         x2=x2+xa(i)*xa(i)
         y2=y2+ya(i)*ya(i)
         xy=xy+xa(i)*ya(i)
      End Do
      dd=1./float(n)
      x = x*dd
      y = y*dd
      x2=x2*dd
      y2=y2*dd
      xy=xy*dd
c
      Return
      End     !compmom
c
c
      subroutine switch(x,px,y,py,g,n1,n2)
c     =================================================================
c     switch two particles
c     -----------------------------------------------------------------
c
      integer n1,n2
      real*8 x(*),px(*),y(*),py(*),g(*),tmp
c
      tmp=x(n1)
      x(n1)=x(n2)
      x(n2)=tmp
      tmp=px(n1)
      px(n1)=px(n2)
      px(n2)=tmp
      tmp=y(n1)
      y(n1)=y(n2)
      y(n2)=tmp
      tmp=py(n1)
      py(n1)=py(n2)
      py(n2)=tmp
      tmp=g(n1)
      g(n1)=g(n2)
      g(n2)=tmp
      return
      end

      subroutine scaledist(x,n,a,b)
c     ==============================================
c     scales distribution to x -> b*(x+a)
c     ----------------------------------------------
c
      real*8 x(*),a,b
      integer i,n
c
      do i=1,n
         x(i)=b*(x(i)+a)
      enddo
      return
      end
c
      subroutine neighbor(x,px,y,py,g,n1,n2,n,iseed)
c     =================================================================
c     search for particle n2 closest to n1
c     -----------------------------------------------------------------
c
      integer n1,n2,n,i,iseed
      real*8 ran1
      real*8 x(*),px(*),y(*),py(*),g(*),r,rmin
c
      rmin=-1.
      n2=0
      do i=1,n
         r=  (x(n1)-x(i))**2*ran1(iseed)+(px(n1)-px(i))**2*ran1(iseed)
         r=r+(y(n1)-y(i))**2*ran1(iseed)+(py(n1)-py(i))**2*ran1(iseed)
         r=r+(g(n1)-g(i))**2*ran1(iseed)
         if (i.ne.n1) then
            if ((rmin.lt.0).or.(r.lt.rmin)) then 
                 n2=i
                 rmin=r
            endif  
         endif
      enddo
      if (n2.eq.0) then
         n2=n1    ! copy himself if only one particle is present
      endif   
      return
      end
c

      SUBROUTINE CUT_TAIL(MP)
c     =======================================================
c     Collimation of the transverse tails
c     ----------------------------------------------------------
c
      include 'genesis.def'
      include 'particle.cmn'
      include 'input.cmn'
c
      INTEGER I,MP
      REAL*8 r
c
      IF (cuttail.le.0) return      
c
      DO i=1,MP
         r=XPART(i)**2+PX(i)**2       
         IF (r.gt.cuttail) THEN
             lost=1
             LOSTID(i)=1
         ENDIF
         r=YPART(i)**2+PY(i)**2       
         IF (r.gt.cuttail) THEN
             lost=1
             LOSTID(i)=1
         ENDIF
      ENDDO
c
      RETURN
      END
