      SUBROUTINE LOADRAD(ISLICE)
c     =========================================================
c     Fills the Array CRFIELD with initial field
c     For start up from noise this should be small
c     ---------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'field.cmn'
      INCLUDE 'input.cmn'
      INCLUDE 'work.cmn'
      INCLUDE 'io.cmn'
      INCLUDE 'diagnostic.cmn'
      INCLUDE 'time.cmn'
      INCLUDE 'sim.cmn'
      
c
      REAL*8  PRADIN
      INTEGER IX,IREC,islice,ierr
c
      PRADOL=PRAD0          !first halfstep no gain (see DIAGNO)
c
c     Load initial field
c     ------------------------------------------------------------------
c
      if (NFIN.le.0) then
        CALL GAUSS_HERMITE(CRFIELD,PRAD0,zrayl,zwaist,xks) !load gauss-hermite mode
      else
        irec=NSLP-1+ISLICE                 !get record number
        if (itdp.eq.0) irec=1              !scan+ss -> use 1sr record
        ierr=READINPUTFIELD(CRWORK2,irec)  !get field from file
        if (ierr.lt.0) CALL LAST           !stop if error occured
        PRADIN=0.
        DO ix=1,ncar*ncar                  !copy to CRFIELD
           CRFIELD(IX)=CRWORK2(IX)
           PRADIN=PRADIN+DBLE(CRWORK2(ix)*CONJG(CRWORK2(ix)))
        ENDDO 
        PRAD0=PRADIN*(DXY*EEV*XKPER0/XKS)**2/vacimp
        PRADOL=PRAD0
      endif
      RETURN
      END       !of LOADRAD
c
c
      SUBROUTINE GAUSS_HERMITE(cfld,power,zr,zw,rks)
c     =======================================================
c     fills array cfld with the fundamental gauss-hermite mode
c     using the total power, wavenumber rks, rayleigh length zr
c     and waist position zw.
c     --------------------------------------------------------
c
      include 'genesis.def'
      include 'sim.cmn'
      include 'field.cmn'
      include 'input.cmn'

      complex*16 cfld(ncar*ncar),cgauss
      real*8     zscal,xcr,ycr,rcr2,power,zr,zw,rks
      integer    iy,ix,idx
c
      CGAUSS=0.5d0*rKS/DCMPLX(ZR,-ZW)     !see SIEGMAN
      ZSCAL = dSQRT(2.d0*VACIMP*Power/PI *dble(CGAUSS))
     +        *rKS/XKPER0**2/EEV !?
 1    DO IY=1,NCAR
         DO IX=1,NCAR
           idx=(iy-1)*ncar+ix
           XCR=DXY*float(IX-1)/xkw0-DGRID
           YCR=DXY*float(IY-1)/xkw0-DGRID
           RCR2=XCR*XCR+YCR*YCR 
           Cfld(idx)=ZSCAL*CDEXP(-CGAUSS*RCR2)  !Gaussian beam
         End Do    !IX
      End DO       !IY
      return
      end
c


      SUBROUTINE LOADSLPFLD(nslp)
c     =========================================================
c     Fills the Array CRTIME with a seeding field for the first 
c     slice.
c     ---------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'field.cmn'
      INCLUDE 'work.cmn'
      INCLUDE 'io.cmn'
c
      INTEGER IX,islp,nslp,ierr
c
      IF (ITDP.EQ.0) RETURN
c
c     Seding of the random number generator
c     ----------------------------------------------------------------
      if (NFIN.LE.0) THEN
         CALL GAUSS_HERMITE(CRWORK3,PRAD0,zrayl,zwaist,xks) !load gauss-hermite mode
      endif

      do islp=0,nslp-1
         if (NFIN.gt.0) then
            ierr=READINPUTFIELD(CRWORK2,islp+1)  !get field from file
            if (ierr.lt.0) call last
            do ix=1,ncar*ncar
               crwork3(ix)=crwork2(ix)
            enddo
         endif   
         call PUSHTIMEREC(CRWORK3,ncar,nslp-1-islp)
      enddo   
      return
      end
c 
c
      FUNCTION READINPUTFIELD(cin,irec)
c     ==============================================
c     read field from input file
c     ----------------------------------------------
c
      include 'genesis.def'
      include 'io.cmn'
      include 'input.cmn'
      include 'sdds.cmn'
      INCLUDE "SDDSFortran.h"
c
      COMPLEX*8 cin(NCAR*NCAR)
      integer ix,irec
c
      INTEGER IY,i,j,pages,rows
      DIMENSION column1(NCAR*NCAR), column2(NCAR*NCAR)
      REAL*8 column1, column2
c
      READINPUTFIELD=NOERR
      if (SDDS .EQ. 0) then
         READ(NFIN,REC=irec,ERR=1) (Cin(IX),IX=1,NCAR*NCAR)
      else
         pages = IREC - SDDSNFINCurrentPage
         if (pages .LT. 0) then
            write (*,*) "Problem reading field file"
            stop 1
         end if
         do j = 1, pages
            i = SDDS_ReadPage_F(1)
            if (i .EQ. 0) then
               call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
               stop 1
            end if
            if (i .EQ. -1) then
               call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
               stop 1
            end if
         end do
         SDDSNFINCurrentPage = SDDSNFINCurrentPage + pages
         rows = SDDS_RowCount_F(1)
         if (rows .NE. NCAR*NCAR) then
            write (6,*) "invalid number of rows in the field file"
            stop 1
         end if
         i = SDDS_GetColumnData_F(1,SDDSNFINRealIndex,column1,rows,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         i = SDDS_GetColumnData_F(1,SDDSNFINImagIndex,column2,rows,
     +        SDDS_DOUBLE)
         if (i .LT. 0) then
            call SDDS_PrintErrors_F(SDDS_VERBOSE_PrintErrors)
            stop 1
         end if
         do i = 1,rows
            Cin(IY+i) = DCMPLX(column1(i),column2(i))
         end do
      endif
      return
 1    READINPUTFIELD=PRINTERR(ERRREADFLD,FIELDFILE)
      return
      end


      SUBROUTINE SWAPFIELD(ISLP)
c     ========================================
c     swap current field with then time-record
c     ----------------------------------------
c
      include 'genesis.def'
      include 'input.cmn'
      include 'field.cmn'
      include 'work.cmn'
c
      integer islp,it
c
      DO it=1,NCAR*NCAR
         CRWORK3(it)=CRFIELD(it)
      enddo
      CALL PULLTIMEREC(CRFIELD,ncar,islp)
      CALL PUSHTIMEREC(CRWORK3,NCAR,ISLP)
      RETURN
      END ! SWAPFIELD
