      Subroutine MAGFIELD(xkw0)
c     ==================================================================
c     calculates the magnetic structure of the undulator
c     Outputs: Awz   - wiggler field on axis
c              Awdz  - artificial K-parameter for matching of drifts 
c              Awerr - kick of momentum due to field errors
c              Qx    - focusing strength in x (>0 -> focusing)
c              Qy    - focusing strength in y
c
c     working arrays: P1 - readin quadrupole field
c                     k2gg - quad offset in x         
c                     k3gg - quad offset in y         
c                     k2pp - correction in x         
c                     k3pp - correction in x         
c
c     ------------------------------------------------------------------
c
      INCLUDE  'genesis.def'
      INCLUDE  'input.cmn'
      INCLUDE  'magnet.cmn'
c
      INTEGER NFL,NDL,NDRL,NFODOL,N1ST,NLAT,i1,i2,i,ISKIP
      INTEGER imz(9),NWIGZ,ISERR,NSL
      REAL*8 NORM,RN,RNX,RNY,ROLD,xkw0,atmp
      REAL*8 CORX(NZMAX),CORY(NZMAX),DQFX(NZMAX),DQFY(NZMAX)
c
c=======================================================================
c Set default version number for external magnetic file.
c
      MAGVERSION = 1.0        ! Version of magnetic file
      AWERR = 0               ! Wiggler errors not included
      QDXERR = 0              ! Quad x-offset not included
      QDYERR = 0              ! Quad y-offset not included
      TAPINCL = 0             ! Tapering not included
c
c=======================================================================
c
c     clear arrays
c
      DO i=1,9
         imz(i)=0
      ENDDO 
      DO i=1,NZMAX
         AWZ(i)=0.
         AWDZ(i)=0.
         QFLD(i)=0.
         SOLZ(i)=0.
         AWERX(i)=0.
         AWERY(i)=0.
         DQFX(i)=0.
         DQFY(i)=0.
         CORX(i)=0.
         CORY(i)=0.
      ENDDO   
c
c     read external file
c
      IF (MAGIN.NE.0) CALL MAGREAD(DQFX,DQFY,CORX,CORY,imz)
c
c     Estimate length of the undulator
c
      NSTEPZ=0
      NLAT=1
      NFODOL=NINT((FL+DL+2.*DRL)/DELZ)    !steps of internal focusing lattice
      if (NFODOL.le.0) NFODOL=1
      NWIGZ=NINT(NWIG/DELZ)               !steps of internal undulator section 
      if (imz(4).gt.0) NFODOL=imz(4)      !structure defined in input file 
      if (imz(1).gt.0) then
         NSEC=1
         NWIGZ=imz(1)
         NSTEPZ=imz(i)                    !AW0 in input file determines the total length
      endif
c
      if (NSTEPZ.LE.0) then               !length yet not defiened
         if (NFODOL.ge.NWIGZ) then
            NLAT=1                        !# of lattice per section
         ELSE
            NLAT=NWIGZ/NFODOL
            if (MOD(NWIGZ,NFODOL).GT.0) NLAT=NLAT+1
         endif   
         NSTEPZ=NSEC*NLAT*NFODOL
      endif   
      if (ZSTOP.LE.0) ZSTOP=NSTEPZ*DELZ*XLAMD
      if (ZSTOP.LE.(NSTEPZ*DELZ*XLAMD)) NSTEPZ=ZSTOP/XLAMD/DELZ
      if (NSTEPZ.GT.NZMAX) then
         i=PRINTERR(ERRARRBND,'Undulator Field')
         NSTEPZ=NZMAX
         zstop=NSTEPZ*DELZ*XLAMD
      endif   
c
      if(MAGIN.NE.0) call chk_maglen(imz,NSTEPZ)     ! check whether input file is long enough
c
c     generatic magnetic field
c
      NFL=NINT(FL/DELZ)            
      NDL=NINT(DL/DELZ)
      NDRL=NINT(DRL/DELZ)
      N1ST=NINT(F1ST/DELZ)+1      
      NSL=NINT(SL/DELZ)
      IF (imz(9).gt.0) NSL=imz(9)
c
      Do i1=1,NSEC
         do i2=1,NFODOL*NLAT
            i=i2+(i1-1)*NLAT*NFODOL
            if (i.le.nstepz) then               !check for array boundary
c
c   If field not externally defined OR if no taper defined
c
              if ((imz(1).le.0).OR.(TAPINCL.EQ.0)) then   !main undulator field
                 if (i2.le.NWIGZ) then
                    ATMP=AW0
                    if (imz(1).gt.0) ATMP=AWZ(i)
                    AWZ(i)=FAW0(i,WCOEFZ,DELZ,XLAMD,ATMP,NSTEPZ,TAPINCL)
                 endif   
              endif
c
              if (imz(2).le.0) then             !drif section 
                if (AWZ(i).lt.TINY) AWDZ(i)=AWD
              endif
c
              IF (N1ST.GT.NFODOL) N1ST=1        !check for overflow of counter
              if (imz(4).le.0) then             !quadrupole field 
                 IF (N1ST.LE.NFL) Qfld(I)=QUADF     
                 IF ((N1ST.GT.(NFL+NDRL)).AND.
     c               (N1ST.LE.(NFL+NDRL+NDL))) Qfld(I)=QUADD
              endif 
              N1ST=N1ST+1
c
              if (imz(9).le.0) THEN             !solenoid field
                  IF (i2.LE.NSL) SOLZ(i)=SOLEN 
              ENDIF                            
            endif  
         enddo   
      enddo
c
      if ((imz(1).le.0).OR.(TAPINCL.EQ.0)) then   ! Has taper been made
         IF(WCOEFZ(2).NE.0) TAPINCL = 1           ! Is there actual taper?
      endif
c
c     field errors: skip this part if AWERR<>0 OR if DELAW=0 (no errors)
c
      IF(AWERR.EQ.0) THEN   ! no/ignore errors in external file
         AWERR=1    ! Set flag for save option afterwards
         rn=0.
         iserr=0
         iskip=1
         if (IERTYP.LT.0) ISKIP=2                  !correlated error
         if (IERTYP.ne.0) then
           do i=1,nstepz
              if (iserr.eq.0) then
                 IF (ABS(IERTYP).EQ.1) THEN    
                    RN=DELAW*(2.D0*RAN1(ISEED)-1.d0)*DSQRT(3.D0) !uniform error distribution
                 ELSE
                    RN=DELAW*GASRAN(ISEED)*DSQRT(2.D0)           !gaussian error distribution 
                 ENDIF
                 ISERR=ISKIP 
              ENDIF
              awz(i)=awz(i)*(1.+RN)
              iserr=iserr-1
           enddo   
         endif   
      ENDIF     ! AWERR <> 0
c
      NORM=DSQRT(2.d0)          !normalization for the kick due to field errors
      IF (IWITYP.NE.0) NORM=1. 
      if (imz(2).le.0) then
        DO i=2,NSTEPZ
           IF ((AWZ(i-1).GT.TINY).AND.(AWZ(i).GT.TINY)) THEN
              AWERX(I)=AWERX(i)+(-1.)**i*(AWZ(I)-AWZ(i-1))*NORM/PI
           ENDIF
        ENDDO
      endif  
c
      do i=1,nstepz
         AWERY(i)=AWERX(i)*DBLE(IWITYP) !is nonzero for helical undulators
      enddo   
c
c     scale quadrupole & solenoid field + quad offsets
c     QDXERR and QDYERR can both be zero or just one of them.... 
c     If their product is zero, check the errors
c
      rold=0.
      RNX=0.
      RNY=0.
      do i=1,nstepz
         qfld(i)=qfld(i)*586.0
         solz(i)=solz(i)*586.0
           IF ((DABS(qfld(i)).gt.tiny).and.(rold.lt.TINY)) then
             IF(QDXERR.EQ.0) RNX=QFDX*(2.*RAN1(ISEED)-1)   !offset
             IF(QDYERR.EQ.0) RNY=QFDY*(2.*RAN1(ISEED)-1)
           endif  
           DQFX(i)=rnx       !doen't matter whether 
           DQFY(i)=rny
           ROLD=qfld(i)
      enddo   
c
      IF((QDXERR*QDYERR.EQ.0).AND.(ABS(QFDX)+ABS(QFDY).GT.0)) THEN
         IF(QFDX.NE.0) QDXERR = 1     ! errors in X generated
         IF(QFDY.NE.0) QDYERR = 1     ! errors in Y generated
      ENDIF
c  
c     write file with field description.
c
      IF (MAGOUT.NE.0) CALL MAGWRITE(DQFX,DQFY,CORX,CORY,XKW0)
c
c     Combine kicks off both planes
c
      DO i=1,NSTEPZ
         AWERX(i)=AWERX(i)+(DQFX(i)*QFLD(i)+CORX(i))/XKW0 !add kick of quad offset
         AWERY(i)=AWERY(i)+(DQFY(i)*QFLD(i)+CORY(i))/XKW0 !to kick of field errors
      ENDDO
c
c     needed for output before 1st step
c
      AWZ(0)=AWZ(1)
      QFLD(0)=QFLD(1)
c
      RETURN
      END
c
c
      Function FAW0(I,WCOEFZ,DELZ,XLAMD,AW0,NSTEPZ,TAPINCL)
c     ==================================================================
c     The wiggler amplitude profile along the undulator
c     The passed variable is the integration step istepz
c     This routine is called only once at the beginning
c
c     The tapering depends on WCOEFZ:
c             WCOEFZ(1): taper start location in Z
c             WCOEFZ(2): gradient of FIELD taper
c             WCOEFZ(3): the type of taper
c                             =0 -> no taper
c                             =1 -> linear taper
c                             =2 -> quadratic taper
c     ------------------------------------------------------------------
c
      implicit NONE
c
      REAL*8 z,TAPLEN,pctap,fac,wcoefz(*),DELZ,XLAMD,AW0,FAW0
      INTEGER i,NSTEPZ,TAPINCL
c
      Z = DBLE(i)*DELZ*XLAMD-WCOEFZ(1)   !position relative to taper start
      FAW0=AW0
      IF ((Z.LE.0.).OR.(WCOEFZ(3).EQ.0.)) RETURN !before taper or no taper at all 
c
c    If external magnetic file includes taper, don't do anything
c
      IF(TAPINCL.NE.0) RETURN
c
      PCTAP= WCOEFZ(2)        !taper gradient
      TAPLEN=DBLE(NSTEPZ)*DELZ*XLAMD-WCOEFZ(1)  !Taper length
c
c     taper models
c     ------------------------------------------------------------------
c
      If (WCOEFZ(3).lt.1.1d0)Then
         FAC=1.d0-PCTAP*Z/TAPLEN           !linear taper
      ELSE
        if (WCOEFZ(3).lt.2.1d0) then
          FAC=1.d0-PCTAP*Z*Z/TAPLEN/TAPLEN  !quadratic taper
        else
          fac=1
        endif
      ENDIF
      FAW0=FAC*FAW0  
c
      Return
      End     !FAW0
c
c
      Function FAW2(I,X,Y)
c     ==================================================================
c     Calculation of the square of the off-axis wiggler field at step I
c     The dependency on x and y is given by the wiggler type
c     ------------------------------------------------------------------
c
      INCLUDE  'genesis.def'
      INCLUDE  'magnet.cmn'
      include  'input.cmn'
c
      REAL*8 x,Y
      INTEGER I
c
      FAW2 = AWZ(I)*AWZ(I)*(1.d0+XKX*X*X+XKY*Y*Y)
c
      Return
      End     !FAW2
c
c
      SUBROUTINE MAGWRITE(DQFX,DQFY,CORX,CORY,XKW0)
c     ===================================================================
c     Output of the used magentic field
c     -------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      include 'magnet.cmn'
      include 'input.cmn'
c
      REAL*8 DQFX(*),DQFY(*),corx(*),cory(*),rold,rcur,XKW0
      INTEGER i,ic,k,nmout,nr
      CHARACTER*3  cid(9)
c
      cid(1)='AW '
      cid(2)='DP '
      cid(3)='QF '
      cid(4)='QX '
      cid(5)='QY '
      cid(6)='AD '
      cid(7)='SL '
      cid(8)='CX '
      cid(9)='CY '
c
cbart      NMOUT=OPENTEXTFILE(MAGINFILE,'UNKNOWN',8)
      NMOUT=OPENTEXTFILE(MAGOUTFILE,'UNKNOWN',8)
      if (NMOUt.LT.0) RETURN

c
c  Write header of the file
c
      WRITE(NMOUT,50) '# Header is included'
      IF(MAGVERSION.GT.1.02) 
     +   WRITE(NMOUT,40) '? VERSION=',MAGVERSION,' including new format'
      IF(TAPINCL.NE.0) 
     +   WRITE(NMOUT,45) '? TAPINCL=',TAPINCL,':tapering included'
      IF(QDXERR.NE.0) 
     +   WRITE(NMOUT,45) '? QDXERR=',QDXERR,': error not overwritten'
      IF(QDYERR.NE.0) 
     +   WRITE(NMOUT,45) '? QDYERR=',QDYERR,':error not overwritten'
      IF(AWERR.NE.0) 
     +   WRITE(NMOUT,45) '? AWERR=',AWERR,': error not overwritten'
c
      do k=1,9
c         i=0
c         ic=0
         i=1
         ic=1
         nr=0
         rold=AWZ(i)
cbart         if (k.eq.2) rold=AWERX(i)
         if (k.eq.2) rold=0.D0
         if (k.eq.3) rold=QFLD(i)/586.0
         if (k.eq.4) rold=DQFX(i)
         if (k.eq.5) rold=DQFY(i)
         if (k.eq.6) rold=AWDZ(i)
         if (k.eq.7) rold=SOLZ(i)/586.0
         if (k.eq.8) rold=CORX(i)
         if (k.eq.9) rold=CORY(i)
 1       i=i+1
         ic=ic+1
         rcur=AWZ(i)
cbart         if (k.eq.2) rcur=AWERX(i)
         if (k.eq.2) rcur= 0.D0
         if (k.eq.3) rcur=QFLD(i)/586.
         if (k.eq.4) rcur=DQFX(i)
         if (k.eq.5) rcur=DQFY(i)
         if (k.eq.6) rcur=AWDZ(i)
         if (k.eq.7) rcur=SOLZ(i)/586.
         if (k.eq.8) rcur=CORX(i)
         if (k.eq.9) rcur=CORY(i)
         if (i.GT.nstepz) THEN
            IF (IC.GT.2) THEN 
               IF(MAGVERSION.GT.1.01) THEN
                  Write(NMOUT,60) cid(k),rold,DELZ*XLAMD,ic-1,nr
                  nr=0
                  ELSE
                  Write(NMOUT,30) cid(k),rold,DELZ*XLAMD,ic-1
               ENDIF
            ENDIF
            goto 2
         ENDIF
         IF (DABS(rcur-rold).GT.TINY) THEN
            IF(MAGVERSION.GT.1.01) THEN
               if(ABS(rold).LT.TINY) THEN
                  nr=ic-1
                  ELSE
                  Write(NMOUT,60) cid(k),rold,DELZ*XLAMD,ic-1,nr
                  nr=0
               ENDIF
               ELSE
               Write(NMOUT,30) cid(k),rold,DELZ*XLAMD,ic-1
            ENDIF
            rold=rcur
            ic=1
         ENDIF
         GOTO 1
 2      continue   
      enddo   
c
cbart 2    CLOSE(NMOUT)
      CLOSE(NMOUT)
      RETURN
c
c     format statements
c
  30  Format(A3,1X,1PE14.4,1X,1PE14.4,1X,I4) 
  40  Format(A,1X,1F4.2,1X,A) 
 45   Format(A,1X,I2,1X,A) 
  50  Format(A) 
  60  Format(A3,1X,1PE14.4,1X,1PE14.4,1X,I4,1X,I4) 
      END  
  
c
c
c
      SUBROUTINE MAGREAD(DQFX,DQFY,CORX,CORY,imz)
c     ========================================================================
c     Input of the magnetic field.
c     Only those parameters are replace which are included in the list
c     Format : Indicator - Strength - Length - Number
c     Inticator: AW - wiggler field
c                AD - phase matching between drift section 
c                DP - Kick due to field errors 
c                QF - Quadrupole field
c                QX - Quadrupole offset in x
c                QY - Quadrupole offset in y
c                CX - Corrector strength in x
c                CY - Corrector strength in y
c                SL - solenoid field
c
c     Note: DP,CO QX are combined into AWERX. Output is only DP
c     --------------------------------------------------------------------------
c
      Include 'genesis.def'
      INCLUDE 'magnet.cmn'
      INCLUDE 'input.cmn'
c
      CHARACTER*30 cmagtype(9),cin
      CHARACTER*255 line
      REAL*8 r1,r2,DQFX(*),DQFY(*),CORX(*),CORY(*)
      REAL*8 r3,values(4),val
      INTEGER i,nr,imz(9),IMZB(9),IDUM
      INTEGER LOOP,loopcnt,j,k,ntemp,NMIN
      INTEGER nloop,ninfo,nr2
      INTEGER int_version,ncol,ierr,nlen,idx
c
      cmagtype(1)='Undulator Field'
      cmagtype(2)='Drift Section'
      cmagtype(3)='Field Errors'      ! not used anymore
      cmagtype(4)='Quadrupole Field'
      cmagtype(5)='Quadrupole Offset in X'
      cmagtype(6)='Quadrupole Offset in Y'
      cmagtype(7)='Orbit Correction in X'
      cmagtype(8)='Orbit Correction in Y'
      cmagtype(9)='Solenoid Field'
c
      nlen=len(line)  ! line size can be easily changed
      loop=0
      loopcnt=0
      int_version=2   ! check if version set to 1.0
      ncol=3          ! default older version to read 3 numbers after the identifier
c
c     open file 
c
      NMIN=OPENTEXTFILE(MAGINFILE,'old',8)
      if (NMIN.LT.0) RETURN
c
 1    READ(NMIN,1000,END=2,ERR=100) line    ! read line
c   
      call touppercase(line)                !convert to upper case
      call getfirstchar(line,idx)           !get index of first non-space char. 
c
      if (idx.eq.0) goto 1                  !empty line
      if (line(idx:idx).eq.'#') goto 1      !comment line
      if (line(idx:idx).eq.'?') then        !information line
         call getmagfileinfo(line(idx+1:nlen))
         IF(MAGVERSION.lt.1.01) then
             NCOL=3
             int_version=1
         ELSE
             NCOL=4                         !read 4 number at version >= 1.02
         endif
         goto 1
      endif
      if (line(idx:idx).eq.'!') then        ! start/end loop structure
         line=line(idx+1:nlen)
         call getfirstchar(line,idx)
         if (idx.eq.0) then   
            i=PRINTERR(ERRINWARN,'Empty line behind "!": ignored')
            goto 1
         ENDIF
c
c Check for start of loop: set loop counter
c
         nloop=index(line,'LOOP')
         if (nloop.eq.0) then
            ierr=PRINTERR(ERRINWARN,'Undefined command in MAGINFILE')
            goto 1
         endif
         if (nloop.eq.idx) then
            if(loop.EQ.1) THEN ! start next loop before ending previous 
               nloop=PRINTERR(ERRINWARN, 'Illegal ending of loop') ! forced stop
               close(nmin)
               call last
            ENDIF     
            ierr=extractnumber(line(idx+4:nlen),val)
            if (ierr.lt.0) then
               i=PRINTERR(ERRINWARN,'Undefined LOOP argument')
               goto 1
            else
               loopcnt=int(val)
            endif
            if (loopcnt.gt.1) then
               loop=1
               do k=1,9
                  imzb(k)=imz(k)
               enddo 
            endif  
            GOTO 1
         endif
c
c Check for loop ending: reset loopcounter and copy loop
c
         nloop=index(line,'ENDLOOP')
         if((nloop.eq.idx).AND.(loop.EQ.1)) THEN        ! a loop is ended
            DO j=2,loopcnt   ! copy filed loopcnt-1 times
              do k=1,9 
                 ntemp=imz(k)-imzb(k)
                 if ((imz(1)+ntemp).gt.nzmax) then
                    idum=PRINTERR(ERRARRBND,cmagtype(k))
                 else
                   DO i=1,ntemp
                     if (k.eq.1) AWZ(imz(1)+i)=AWZ(imzb(1)+i)
                     if (k.eq.2) AWDZ(imz(2)+i)=AWDZ(imzb(2)+i)
cbart                     if (k.eq.3) AWERX(imz(3)+i)=AWERX(imzb(3)+i)
                     if (k.eq.4) QFLD(imz(4)+i)=QFLD(imzb(4)+i)
                     if (k.eq.5) DQFX(imz(5)+i)=DQFX(imzb(5)+i)
                     if (k.eq.6) DQFY(imz(6)+i)=DQFY(imzb(6)+i)
                     if (k.eq.7) CORX(imz(7)+i)=CORX(imzb(7)+i)
                     if (k.eq.8) CORY(imz(8)+i)=CORY(imzb(8)+i)
                     if (k.eq.9) SOLZ(imz(9)+i)=SOLZ(imzb(9)+i)
                   ENDDO
                   imz(k) = imz(k)+ntemp
                   imzb(k)=imzb(k)+ntemp
                 endif
              enddo
            END DO
            loop=0     !finish the round of copying
            loopcnt=0
            GOTO 1
         ENDIF
      endif
c
c     process input line
c
      cin=line(idx:idx+1)   !get identifier
      k=-1
      if (cin(1:2).eq.'AW') k=1 !main wiggler field
      if (cin(1:2).eq.'AD') k=2 !art. wiggler field between section
c      if (cin(1:2).eq.'DP') k=3 !field error
      if (cin(1:2).eq.'QF') k=4 !quadrupole field strength
      if (cin(1:2).eq.'QX') k=5 !quadrupole offset in x
      if (cin(1:2).eq.'QY') k=6 !quadrupole offset in y
      if (cin(1:2).eq.'CX') k=7 !correcter strength in x
      if (cin(1:2).eq.'CY') k=8 !corrector strength in y
      if (cin(1:2).eq.'SL') k=9 !solenoid strength
      if (k.lt.0) then
         ierr=PRINTERR(ERRCONV,cin(1:2))
         goto 1
      endif
c      
      line=line(idx+2:nlen)
      call getfirstchar(line,idx) !eliminate spaces in the beginning
      if (idx.gt.0) line=line(idx:nlen) 
      ierr=extractval(line,values,ncol)
      if (ierr.lt.0) then
         ierr=PRINTERR(ERRCONV,line)
         goto 1
      endif
      r1=values(1)              !strength/offset
      r2=values(2)              !unit length
      nr=nint(values(3))         !lenght in unit length
c
c     filling the arrays
c
      if (magversion.ge.1.01) then
         nr2=nint(values(4)) !separation to prev. element       
         r3=r2*nr2
         nr2 = nint(r3/delz/xlamd)
         if(abs(nr2-r3/delz/xlamd).gt.SMALL) then
            i=PRINTERR(ERRMAGRND,line) ! Different error needed ?????????
         ENDIF 
         if ((imz(k)+nr2).GT.NZMAX) THEN
            idum=PRINTERR(ERRARRBND,cmagtype(k))
         else
            imz(k)=imz(k)+nr2
         ENDIF 
      ENDIF
c
      r2=r2*nr                !full length of this section
      nr=nint(r2/delz/xlamd)  !# int. steps for this section
      if(abs(nr-r2/delz/xlamd).gt.SMALL) then
        i=PRINTERR(ERRMAGRND,line) 
      ENDIF 
      if ((imz(k)+nr).GT.NZMAX) THEN
         idum=PRINTERR(ERRARRBND,cmagtype(k))
      else
         DO i=1,nr
            if (k.eq.1) AWZ(imz(1)+i)=r1
            if (k.eq.2) AWDZ(imz(2)+i)=r1
c            if (k.eq.3) AWERX(imz(3)+i)=r1
            if (k.eq.4) QFLD(imz(4)+i)=r1
            if (k.eq.5) DQFX(imz(5)+i)=r1
            if (k.eq.6) DQFY(imz(6)+i)=r1
            if (k.eq.7) CORX(imz(7)+i)=r1
            if (k.eq.8) CORY(imz(8)+i)=r1
            if (k.eq.9) SOLZ(imz(9)+i)=r1
         ENDDO
         imz(k)=imz(k)+nr
      endif
      goto 1      ! read new line
c
c     final processing + closing files
c
 2    IF(int_version.EQ.2) MAGVERSION = 2.0
      if(loop.EQ.1) ninfo=
     c       PRINTERR(ERRINWARN, '"LOOP" not terminated: no loop') 
      CALL CLOSEFILE(NMIN)
      RETURN 
c
c     Error
c
 100  idum=PRINTERR(ERRREAD,MAGINFILE)
      goto 1
c
c     format statements
c
 1000 FORMAT(a)
c
      END

c
      SUBROUTINE CHK_maglen(imz,NSTEPZ)
c     ===================================================================
c     checks whether the user supplied file for the description of the
c     magnetic fields is incomplete
c     -------------------------------------------------------------------
c
      include 'genesis.def'
c
      character*30   cmagtype(9)
      integer i,j,imz(9),NSTEPZ
c
      cmagtype(1)='Undulator Field'
      cmagtype(2)='Drift Section'
cbart      cmagtype(3)='Field Errors'
      cmagtype(4)='Quadrupole Field'
      cmagtype(5)='Quadrupole Offset in X'
      cmagtype(6)='Quadrupole Offset in Y'
      cmagtype(7)='Orbit Correction in X'
      cmagtype(8)='Orbit Correction in Y'
      cmagtype(9)='Solenoid Field'
c
      Do i=1,9
         if ((imz(i).GT.1).AND.(imz(i).LT.NSTEPZ)) THEN
            j=PRINTERR(ERRMAGSHORT,cmagtype(i))
         ENDIF
      ENDDO
      RETURN
      END ! of chk_maglen
c
c
      subroutine getmagfileinfo(line)
c     =================================================================
c     extract information from BEAMFILE
c     -----------------------------------------------------------------
c
      include 'genesis.def'
      include 'magnet.cmn'
c
      character*(*) line
      integer n,i,idx,ierr
      real*8  val
c
      n=len(line)
      call getfirstchar(line,idx) ! get first character should be identical to INDEX !!
c
c     version number
c
      i=index(line,'VERSION') !check for version number
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+7:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in MAGINFILE')
        else
           magversion=val
        endif
        return
      endif 
c
c     Taper
c
      i=index(line,'TAPINCL')
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+7:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in MAGINFILE')
        else
           tapincl=nint(val)
        endif
        return
      endif 
c
c     errors
c
      i=index(line,'AWERR') 
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+5:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in MAGINFILE')
        else
           AWERR=nint(val)
        endif
        return
      endif 
c
c     quadrupole offset in x
c
      i=index(line,'QDXERR') 
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+6:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in MAGINFILE')
        else
           QDXERR=nint(val)
        endif
        return
      endif 
c
c
c     quadrupole offset in y
c
      i=index(line,'QDYERR') 
      if ((i.gt.0).and.(i.eq.idx)) then
        ierr=extractnumber(line(i+6:n),val)
        if (ierr.lt.0) then
           i=PRINTERR(ERRINWARN,
     c               'Unrecognized information line in MAGINFILE')
        else
           QDYERR=nint(val)
        endif
        return
      endif 
c
c     unrecognized
c
      i=PRINTERR(ERRINWARN,'Unrecognized information line in BEAMFILE')
      return
      end
c
