c     ############################################################# 
c
c             DEUSQUE DIXIT FIAT LUX ET FACTA EST LUX
c 
c     #############################################################
c
c     GENESIS 1.3 is a time dependent FEL code. The very basic structure
c     and the naming of most variables are taken from its precessor 
c     TDA3D.
c    
c     Particle Integration:  Runge-Kutta 4th order
c     Field Integration:     Alternating Direction Implicit-Methode   
c     Space Charge Field:    Inversion of Tridiagonal Matrix 
c 
c     GENESIS 1.3 is a CPU and Memory expensive program and migth 
c     exceed the requirement on older platforms. It can be partially
c     reduced by excluding time-dependent simulation or, as an
c     alternative, using a scratch file for most of the stored
c     information.
c
c
c     Author:     Sven Reiche     (Main Algorithm, Version 1.0)
c     Co-Author:  Bart Faatz      (External Modules, Version 1.0)
c                 Pascal Elleaume (Quantum Fluctuation)
c                 Micheal Borland (SDDS)
c                 Robert Soliday  (SDDS)
c   
c     ###############################################################
c    
c     GENESIS 1.3 was written in 1997 at DESY, Hamburg as a part
c     of my  Ph.D.-thesis. I intended as an open-source project.
c     I'm willing to discuss with others about modifications,
c     found bugs and extensions which migth become official in the
c     release of the next version of GENESIS 1.3
c 
c     The current contact is 
c              reiche@stout.physics.ucla.edu
c        
c         
c     Sven Reiche, UCLA, 08/24/00
c
c     #################################################################
c     ------------------------------------------------------------------
c     MAIN UNIT
c     ------------------------------------------------------------------
      PROGRAM GENESIS
c
      INCLUDE 'genesis.def'      !Insert common blocks from file
      INCLUDE 'sim.cmn'
      INCLUDE 'input.cmn'
      include 'time.cmn'
      include 'field.cmn'
      include 'work.cmn'
      include 'magnet.cmn'
      include 'sdds.cmn'
c
c      include 'particle.cmn'
c      real*8 rdis,rr
c      integer ip

      Integer ISLP,ISEP,LSTEPZ,ISTEPZ,ISLICE
c
      call initio     !open files, read in input
c
      call initrun    !initialize simulation 
      CALL OUTGLOB    !output of global parameter (t-independent)
c
c     Start loop for each slice
c
      DO ISLICE=1,NSLICE   !Loop for each slice
c
c     Initial loading
c 
        ISTEPZ=0
c
        CALL DOSCAN(ISLICE)           !update scan value
        CALL DOTIME(ISLICE)           !calculate time-dependent parameters
c      
        CALL LOADRAD(ISLICE)          !radiation field loading
        CALL LOADBEAM(ISLICE,xkw0)    !particle loading
        CALL CHK_LOSS                 !remove cut particle
c
c       not official begin
c
c        itap=1
c        rdis=(gamma0-gamma(1))**2+(THETA(1)-wcoefz(1))**2
c        do IP=2,npart
c           rr=(gamma0-gamma(ip))**2+(THETA(ip)-wcoefz(1))**2
c           if (rr.lt.rdis) then
c                 rdis=rr
c                 itap=ip
c           endif
c        enddo
c 
c       not official end
c
        CALL OUTPUT(ISTEPZ,islice,xkw0)

c
c       Propagate beam for NU wiggler periods
c
        DO ISLP=0,MAX(NSLP,1)-1                !Loop over slippage (advance field)
           LSTEPZ=NSEP
           IF (ISLP.EQ.(MAX(NSLP,1)-1))  LSTEPZ=NSTEPZ-ISLP*NSEP
           DO ISEP=1,LSTEPZ                    !Loop 'steady state' simulation
              ISTEPZ=ISTEPZ+1
              CALL STEPZ(ISTEPZ,xkw0)          !Advance one step in Z         
              CALL OUTPUT(ISTEPZ,islice,xkw0)
           End DO 
           IF (ITDP.NE.0) CALL SWAPFIELD(ISLP) !advance field in time-dep sim
        End Do             
        CALL OUTDUMP(ISLICE,xkw0)              !dump rad + part if needed
        if (SDDS .EQ. 1) then
           CALL WRITENOUT
        endif
      END DO       
c
      CALL LAST   !end run
      STOP
      End     ! Main
c

