      REAL*8 Function gasham(j)
c     ==================================================================
c     Gaussian Hammersley sequence
c     reinitializable
c     ------------------------------------------------------------------
c
      INTEGER j,jmax,icall,iset,jd
      Parameter (jmax=26)
      Real*8 v1,v2,hammv,r,fac,gset
      Dimension iset(jmax),gset(jmax)
      Data icall/0/,iset/jmax*0/,gset/jmax*0.d0/
c
      If((icall.eq.0).or.(j.lt.0)) then
	 Do jd=1,jmax
	    iset(jd)=0
	 End Do
	 icall=1
      End If
      If(iset(j).eq.0) then
 1       v1=hammv(j)*2.d0-1.d0
	 v2=hammv(j+1)*2.d0-1.d0
	 r=v1**2+v2**2
	 If(r.ge.1..OR.r.eq.0.)goto 1
	 fac=dsqrt(-2.d0*dlog(1.d0-r)/r)
	 gset(j)=v1*fac
	 gasham =v2*fac
	 iset(j)=1
      else
	 gasham=gset(j)
	 iset(j)=0
      End If
c
      Return
      End     !gasham
c
c
c
      Real*8 function hammv(j)
c     ==================================================================
c     Uniform Hammersley sequence
c     reinitializable
c     ------------------------------------------------------------------
c
      REAL*8 xs,xsi
      INTEGER j,jmax,i,nbase,i1,i2,icall,jd
      Parameter (jmax=26)
      Dimension nbase(jmax),i(jmax),xs(jmax),xsi(jmax)
      Dimension i1(jmax),i2(jmax)
      Data nbase/2.,3.,5.,7.,11.,13.,17.,19.,23.,29.,31.,37.,41.,43.,
     1          47.,53.,59.,61.,67.,71.,73.,79.,83.,89.,97.,101./
      Data icall/0/
      Data i/jmax*0/
c
      If ((icall.eq.0).or.(j.lt.0)) then
	 Do jd=1,jmax
	    i(jd)=0
	 End Do
	 icall=1
         j=ABS(J)
      End If
      xs (j)=0.d0
      xsi(j)=1.0d0
      i(j)= i(j)+1
      i2 (j)= i(j)
 10   xsi(j)=xsi(j)/float(nbase(j))
      i1 (j)= i2(j)/nbase(j)
      xs (j)= xs(j)+(i2(j)-nbase(j)*i1(j))*xsi(j)
      i2 (j)= i1(j)
      If(i2(j).gt.0) goto 10
      hammv=xs(j)
c
      Return
      End     !hammv

c
c
c
      REAL*8 FUNCTION RAN1(IDUM)
c     ==================================================================
c     Random number generator from Numerical Recipes (p. 272f).
c     ------------------------------------------------------------------
c
      INTEGER idum,IM1,IM2,IMM1,IA1,IA2,IQ1,IQ2,IR1,IR2,NTAB,NDIV
      REAL*8  AM,EPS,RNMX
      PARAMETER (IM1=2147483563,IM2=2147483399,AM=1./IM1,IMM1=IM1-1,
     +           IA1=40014,IA2=40692,IQ1=53668,IQ2=52774,IR1=12211,
     +           IR2=3791,NTAB=32,NDIV=1+IMM1/NTAB,EPS=1.2d-40,
     +           RNMX=1.d0-EPS)
      INTEGER IDUM2,J,K,iv(NTAB),iy
      SAVE iv,iy,idum2
      DATA idum2/123456789/,iv/NTAB*0/,iy/0/
c
      IF (IDUM.LE.0) THEN
          IDUM=max(-idum,1)
          IDUM2=IDUM
          DO j=NTAB+8,1,-1
             k=IDUM/IQ1
             IDUM=IA1*(IDUM-k*IQ1)-k*IR1
             IF (IDUM.LT.0) IDUM=IDUM+IM1
             IF (j.LE.NTAB) IV(J)=IDUM
          ENDDO
          iy=iv(1)
      ENDIF
      k=idum/IQ1
      idum=IA1*(IDUM-k*IQ1)-k*IR1
      if (IDUM.LT.0) idum=idum+IM1
      k=IDUM2/IQ2
      idum2=IA2*(IDUM2-k*IQ2)-k*ir2
      IF (IDUM2.LT.0) idum2=idum2+IM2
      j=1+iy/NDIV
      IY=iv(j)-idum2
      IV(J)=IDUM 
      IF (IY.LT.1) IY=IY+IMM1
      ran1=min(AM*IY,RNMX)
      RETURN
      END
c
c
      REAL*8 FUNCTION GASRAN(IDUM)
c     ==================================================================
c     Random number generator from Numerical Recipes (p. 272f).
c     ------------------------------------------------------------------
c
      INTEGER iset,idum
      Real*8 v1,v2,ran1,r,fac,gset
c
      If(iset.eq.0) then
 1       v1=ran1(IDUM)*2.d0-1.d0
	 v2=ran1(IDUM)*2.d0-1.d0
	 r=v1**2+v2**2
	 If(r.ge.1..OR.r.eq.0.)goto 1
	 fac=dsqrt(-2.d0*dlog(1.d0-r)/r)
	 gset=v1*fac
	 gasran =v2*fac
	 iset=1
	 else
	 gasran=gset
	 iset=0
      End If
c
      RETURN
      END
c
c
c
      FUNCTION BESSJ0(X)
c     ==================================================================
c     Bessel function J0 - numerical rec.
c     ------------------------------------------------------------------
c
      REAL*8 Y,P1,P2,P3,P4,P5,Q1,Q2,Q3,Q4,Q5,R1,R2,R3,R4,R5,R6,
     *    S1,S2,S3,S4,S5,S6, X, BESSJ0, AX, Z, XX
      DATA P1,P2,P3,P4,P5/1.D0,-.1098628627D-2,.2734510407D-4,
     *    -.2073370639D-5,.2093887211D-6/, Q1,Q2,Q3,Q4,Q5/-.1562499995D-
     *1,
     *    .1430488765D-3,-.6911147651D-5,.7621095161D-6,-.934945152D-7/
      DATA R1,R2,R3,R4,R5,R6/57568490574.D0,-13362590354.D0,651619640.7D
     *0,
     *    -11214424.18D0,77392.33017D0,-184.9052456D0/,
     *    S1,S2,S3,S4,S5,S6/57568490411.D0,1029532985.D0,
     *    9494680.718D0,59272.64853D0,267.8532712D0,1.D0/
      IF(ABS(X).LT.8.)THEN
         Y=X**2
         BESSJ0=(R1+Y*(R2+Y*(R3+Y*(R4+Y*(R5+Y*R6)))))
     *      /(S1+Y*(S2+Y*(S3+Y*(S4+Y*(S5+Y*S6)))))
         ELSE
         AX=ABS(X)
         Z=8./AX
         Y=Z**2
         XX=AX-.785398164
         BESSJ0=SQRT(.636619772/AX)*(COS(XX)*(P1+Y*(P2+Y*(P3+Y*(P4+Y
     *      *P5))))-Z*SIN(XX)*(Q1+Y*(Q2+Y*(Q3+Y*(Q4+Y*Q5)))))
      ENDIF
c
      RETURN
      END
c
c
c
      FUNCTION BESSJ1(X)
c     ==================================================================
c     Bessel function J1 - numerical rec.
c     ------------------------------------------------------------------
c
      REAL*8 Y,P1,P2,P3,P4,P5,Q1,Q2,Q3,Q4,Q5,R1,R2,R3,R4,R5,R6,
     *    S1,S2,S3,S4,S5,S6, X, BESSJ1, AX, Z, XX
      DATA R1,R2,R3,R4,R5,R6/72362614232.D0,-7895059235.D0,242396853.1D0
     *,
     *    -2972611.439D0,15704.48260D0,-30.16036606D0/,
     *    S1,S2,S3,S4,S5,S6/144725228442.D0,2300535178.D0,
     *    18583304.74D0,99447.43394D0,376.9991397D0,1.D0/
      DATA P1,P2,P3,P4,P5/1.D0,.183105D-2,-.3516396496D-4,.2457520174D-5
     *,
     *    -.240337019D-6/, Q1,Q2,Q3,Q4,Q5/.04687499995D0,-.2002690873D-3
     *,
     *    .8449199096D-5,-.88228987D-6,.105787412D-6/
      IF(ABS(X).LT.8.)THEN
         Y=X**2
         BESSJ1=X*(R1+Y*(R2+Y*(R3+Y*(R4+Y*(R5+Y*R6)))))
     *      /(S1+Y*(S2+Y*(S3+Y*(S4+Y*(S5+Y*S6)))))
         ELSE
         AX=ABS(X)
         Z=8./AX
         Y=Z**2
         XX=AX-2.356194491
         BESSJ1=SQRT(.636619772/AX)*(COS(XX)*(P1+Y*(P2+Y*(P3+Y*(P4+Y
     *      *P5))))-Z*SIN(XX)*(Q1+Y*(Q2+Y*(Q3+Y*(Q4+Y*Q5)))))
     *      *SIGN(1.D0,X)
      ENDIF
c
      RETURN
      END
c
c
c
      Integer Function LUF(x,table,n)
c     ==================================================================
c     LUF is a table lookup function that locates a value x between
c     elements of an increasing table of size n.
c     LUF is the value of the index AFTER the table location which
c     X corresponds to.
c     The routine uses a Bisection methode (numerical rec.)
c     The array TABLE must be monotonic
c     LUF=1 or LUF=N+2 is returned to indicate out of range
c     ------------------------------------------------------------------
c
      Real*8 X,TABLE(*)
      INTEGER n,jl,ju,jm
c
      JL=0            !Lower limit
      JU=N+1          !Upper limit
 10   If(JU-JL.GT.1) Then
	 JM=(JU+JL)/2    !Midpoint
	 If((TABLE(N).GT.TABLE(1)).EQV.(X.GT.TABLE(JM))) Then
	    JL=JM
	    Else
	    JU=JM
	 EndIf
	 Goto 10
      EndIf
      LUF=JL+1
c
      Return
      End     !LUF
c
c
c
      SUBROUTINE FOURN(data,nn,ndim,isign)
c     =================================================================
c     multidimensional FFT of complex values (Num. Rec.)
c     Number of elements of DATA must be a power of 2!
c     nn = number of dimension
c     ndim(nn) = elements per dimension
c     isign = 1 FFT, = -1 inverse FFT     
c     -----------------------------------------------------------------
c
      INTEGER isign,ndim,nn(ndim)
      REAL*8 data(*)
      INTEGER i1,i2,i2rev,i3,i3rev,ibit,idim,ifp1,ifp2,ip1,ip2,ip3,k1,
     *k2,n,nprev,nrem,ntot
      REAL*8 tempi,tempr
      DOUBLE PRECISION theta,wi,wpi,wpr,wr,wtemp
      ntot=1
      do 11 idim=1,ndim
        ntot=ntot*nn(idim)
11    continue
      nprev=1
      do 18 idim=1,ndim
        n=nn(idim)
        nrem=ntot/(n*nprev)
        ip1=2*nprev
        ip2=ip1*n
        ip3=ip2*nrem
        i2rev=1
        do 14 i2=1,ip2,ip1
          if(i2.lt.i2rev)then
            do 13 i1=i2,i2+ip1-2,2
              do 12 i3=i1,ip3,ip2
                i3rev=i2rev+i3-i2
                tempr=data(i3)
                tempi=data(i3+1)
                data(i3)=data(i3rev)
                data(i3+1)=data(i3rev+1)
                data(i3rev)=tempr
                data(i3rev+1)=tempi
12            continue
13          continue
          endif
          ibit=ip2/2
1         if ((ibit.ge.ip1).and.(i2rev.gt.ibit)) then
            i2rev=i2rev-ibit
            ibit=ibit/2
          goto 1
          endif
          i2rev=i2rev+ibit
14      continue
        ifp1=ip1
2       if(ifp1.lt.ip2)then
          ifp2=2*ifp1
          theta=isign*6.28318530717959d0/(ifp2/ip1)
          wpr=-2.d0*sin(0.5d0*theta)**2
          wpi=sin(theta)
          wr=1.d0
          wi=0.d0
          do 17 i3=1,ifp1,ip1
            do 16 i1=i3,i3+ip1-2,2
              do 15 i2=i1,ip3,ifp2
                k1=i2
                k2=k1+ifp1
                tempr=sngl(wr)*data(k2)-sngl(wi)*data(k2+1)
                tempi=sngl(wr)*data(k2+1)+sngl(wi)*data(k2)
                data(k2)=data(k1)-tempr
                data(k2+1)=data(k1+1)-tempi
                data(k1)=data(k1)+tempr
                data(k1+1)=data(k1+1)+tempi
15            continue
16          continue
            wtemp=wr
            wr=wr*wpr-wi*wpi+wr
            wi=wi*wpr+wtemp*wpi+wi
17        continue
          ifp1=ifp2
        goto 2
        endif
        nprev=n*nprev
18    continue
      return
      END



