      Subroutine PARTSIM(TGAM,TTHET,DGAM,DTHET)
c     ==================================================================
c     Define the system of ODE of the canonic variables
c     T... are the values of the variables at current position
c     D... are the value of the differentioan equation
c     ------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'particle.cmn'
      INCLUDE 'work.cmn'
c
      COMPLEX*16 CTMP
      INTEGER ip
      REAL*8 TGAM,TTHET,DGAM,DTHET,ztemp1,ztemp2,BTPER0
      Dimension TGAM(*),TTHET(*)
      Dimension DGAM(*),DTHET(*)
c
c     Diff-Eq. for longitudinal variables:
c        - Energy (GAMMA)
c        - Phase (THETA) 
c     ------------------------------------------------------------------
c
      ZTEMP1=-2.d0*XLAMDS/XLAMD
      ZTEMP2=XLAMD/XLAMDS
      Do IP=1,NPART
c
        CTMP=CPART1(IP)*DCMPLX(DCOS(TTHET(IP)),-DSIN(TTHET(IP)))
c
	BTPER0=BTPER(IP)+ZTEMP1*DBLE(CTMP)  !perpendicular velocity
	BTPAR(IP)=DSQRT(1.d0-BTPER0/TGAM(IP)**2)     !parallel velocity
c
	DTHET(IP)=DTHET(IP)+ZTEMP2*(1.-1./BTPAR(IP))+1.      !dTHETA/dz
	DGAM (IP)=DGAM(IP) +DIMAG(CTMP)/BTPAR(IP)/TGAM(IP)-EZ(IP)  !dGAMMA/dz
      End Do       ! IP
      Return
      End     !PARTSIM
c

      Subroutine PARTSORC(istepz)
c     ==================================================================
c     Calculates source term for gamma-theta integration
c     The particle transverse position and the radiation field are taken 
c     at the middle of the integration step.
c     ------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'field.cmn'
      INCLUDE 'input.cmn'
      INCLUDE 'magnet.cmn'
      INCLUDE 'particle.cmn'
      INCLUDE 'sim.cmn'
      INCLUDE 'work.cmn'
c
      COMPLEX*16 CLOCAL
      INTEGER ip,idx,istepz
      REAL*8  zconst,wei,AW2
c
      CALL RPOS(istepz,XPART,YPART)                          !position of particles
c
      ZCONST=XLAMDS/XLAMD*FBESS
      Do IP=1,NPART
        wei=wx(ip)*wy(iP)
        idx=ipos(1,ip)
        CLOCAL=wei*CRFIELD(idx)
        wei=wx(ip)*(1.d0-wy(ip))
        idx=ipos(2,ip)
        CLOCAL=CLOCAL+wei*CRFIELD(idx)
        wei=(1.d0-wx(ip))*wy(iP)
        idx=ipos(3,ip)
        CLOCAL=CLOCAL+wei*CRFIELD(idx)
        wei=(1.d0-wx(ip))*(1.d0-wy(ip))
        idx=ipos(4,ip)
        CLOCAL=CLOCAL+wei*CRFIELD(idx)      !field at particle position
c
        AW2=FAW2(ISTEPZ,XPORB(IP),YPORB(IP))     !square of wiggler field amplitude
        AW2=AW2+AWDZ(ISTEPZ)*AWDZ(ISTEPZ)             !artificial delay in drifts 
        BTPER(IP)=AW2+PX(IP)*PX(IP)+PY(IP)*PY(IP)+1.
        CPART1(IP)=CONJG(CLOCAL)*ZCONST*DSQRT(AW2)
      End Do       ! IP
      Return
      End     !PARTSIM
c

 
