      Subroutine PUSHP(ISTEPZ,xkper0)
c     ==================================================================
c     Advance the particles, using Runge-Kutta (fourth order) method
c     ------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'particle.cmn'
      INCLUDE 'work.cmn'
c
      INTEGER n,istepz
      REAL*8  stpz,xkper0
c
      CALL TRACK(ISTEPZ,xkper0)          !advance particle transversly half integragtion step
c
      CALL PARTSORC(istepz)   !get source term at z0+delz/2
c
c     First step
c     ------------------------------------------------------------------
c
      do n=1,npart    ! clear work arrays
	 k2gg(n)=0.0
	 k2pp(n)=0.0
      end do       ! n
c
C     ODE at Z0
c
      call partsim(gamma,theta,k2gg,k2pp)
c
c     Second step
c     ------------------------------------------------------------------
c
      stpz=0.5*delz*twopi
      do n=1,npart
c K1 = D/2 K2 + K1
	 gamma(n)=stpz*k2gg(n)+gamma(n)
	 theta(n)=stpz*k2pp(n)+theta(n)
c K3 = K2
	 k3gg(n)=k2gg(n)
	 k3pp(n)=k2pp(n)
c K2 = 0
	 k2gg(n)=0.0
	 k2pp(n)=0.0
      end do       ! n

C ODE at Z0+DELZ/2

      call partsim(gamma,theta,k2gg,k2pp)
c
c     Third step
c     ------------------------------------------------------------------
c
      do n=1,npart
c K1 = D/2 K2 + K1
	 gamma(n)=stpz*k2gg(n)+gamma(n)
	 theta(n)=stpz*k2pp(n)+theta(n)
c K1 = -D/2 K3 + K1
	 gamma(n)=-stpz*k3gg(n)+gamma(n)
	 theta(n)=-stpz*k3pp(n)+theta(n)
c K3 = K2/6
	 k3gg(n)=k3gg(n)/6.0
	 k3pp(n)=k3pp(n)/6.0
c K2 = -K2/2
	 k2gg(n)=-k2gg(n)/2.0
	 k2pp(n)=-k2pp(n)/2.0
      end do       ! n
c
C ODE at Z0+DELZ/2
c
      call partsim(gamma,theta,k2gg,k2pp)
c
c     Fourth step
c     ------------------------------------------------------------------
c
      stpz=delz*twopi
      do n=1,npart
c K1 = D K2 + K1
	 gamma(n)=stpz*k2gg(n)+gamma(n)
	 theta(n)=stpz*k2pp(n)+theta(n)
c K3 = -K2 + K3
	 k3gg(n)=-k2gg(n)+k3gg(n)
	 k3pp(n)=-k2pp(n)+k3pp(n)
c K2 = -2 K2
	 k2gg(n)=k2gg(n)*2.0
	 k2pp(n)=k2pp(n)*2.0
      end do       ! n
c
C ODE at Z0+DELZ
c
      call partsim(gamma,theta,k2gg,k2pp)
c
      do n=1,npart
	 gamma(n)=gamma(n)+stpz*(k3gg(n)+k2gg(n)/6.0)
	 theta(n)=theta(n)+stpz*(k3pp(n)+k2pp(n)/6.0)
      end do       ! n
c
      CALL TRACK(ISTEPZ,xkper0)        !advance particle transversly half integragtion step
c
c     wakefields or other energy losses if selected
c
      IF (DEDZ.NE.0.) THEN 
         DO n=1,NPART
            GAMMA(n)=GAMMA(n)+DEDZ
         ENDDO
      ENDIF
c 
c     check for particles outside the gridd  
c
      call chk_loss
c
      return
      end     !pushp
c
c
c
      SUBROUTINE CHK_LOSS
c     ========================================================================
c     checks for lost particles, reorganizing the particle arrays
c     ------------------------------------------------------------------------ 
c
      INCLUDE 'genesis.def'
      INCLUDE 'input.cmn'
      INCLUDE 'particle.cmn'
c
      REAL*8  rtmp
      character*3 closs
      INTEGER DELIP(NPMAX),MPART,j,k,i,idel
c
      IF (LOST.LE.0) RETURN            !no loss-initialized in cut-tail
c
      MPART=NPART/NBINS
      j=0  
c
      DO k=0,NBINS-2
         DO i=1,MPART                         !run over one set of mirror particles
            if (LOSTID(i+k*MPART).NE.0) THEN  !lost ?
               j=j+1                          !count &
               DELIP(j)=i                     !get index
            ENDIF                         
         ENDDO
         DO i=1,j                             !make sure that the found particles
            LOSTID(DELIP(i)+(k+1)*MPART)=0    !are not countet in next set
         ENDDO
      ENDDO
c
      DO i=1,MPART                            !search last set
         if (LOSTID(i+(NBINS-1)*MPART).NE.0) THEN
            j=j+1
            DELIP(j)=i
         ENDIF
      ENDDO
c
      DO i=1,j
         do k=0,NBINS-1
           GAMMA(DELIP(i)+k*MPART)=-1.
         enddo  
      ENDDO
c      
      idel=0          
      DO I=1,NPART 
         IF (GAMMA(I).GT.0.) THEN
            GAMMA(I-idel)=GAMMA(i)  
            THETA(I-idel)=THETA(i)  
            XPART(i-idel)=XPART(i)
            YPART(i-idel)=YPART(i)
            PX(i-idel)=PX(i)
            PY(i-idel)=PY(i)
         ELSE    
            idel=idel+1
         ENDIF
         LOSTID(i)=0   !clear flags of lost particles
      ENDDO
c
      lost=0
c
c     get numbers right
c
      NPART=NPART-NBINS*j 
      XCUREN=XCUREN*FLOAT(NPART)/FLOAT(NPART+NBINS*j)           
      rtmp=1.d0-DBLE(NPART)/DBLE(NPART+NBINS*j)
      IF (rtmp.gt.0.01) then
         WRITE(closs,100) Rtmp*100.
         i=PRINTERR(ERRPARTLOSS,closs)
      ENDIF   
c
 100  FORMAT(F4.0)
      RETURN
      END              !chk_loss
c
c

