      Subroutine RPOS(istepz,XX,YY)
c     ==================================================================
c     Locates the position of the electron on its actual trajectory
c     Apply orbit correction to account for wiggle motion.
c     ------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'field.cmn'
      INCLUDE 'input.cmn'
      INCLUDE 'magnet.cmn'
      INCLUDE 'particle.cmn'
      include 'sim.cmn'
      
c
      REAL*8 x,awtmp,wxlow,wylow,XX(*),YY(*)
      INTEGER ip,ix1,iy1,ix2,iy2,istepz
c
      x=(DBLE(ISTEPZ)+0.5)*DELZ*twopi
c
c     Orbit correction ?
c     -------------------------------------------------------------------
      IF (IORB.EQ.0) THEN
        Do IP=1,NPART                   !no orbit correction
	   XPORB(IP)=XX(IP)
	   YPORB(IP)=YY(IP)
        END DO
      ELSE
        IF (FBESS.LT.1.) THEN
           DO IP=1,NPART                                 !planar undulator
              AWTMP=DSQRT(FAW2(ISTEPZ,XX(IP),YY(IP)))   !aw at particle position
	      XPORB(IP)=XX(IP)-AWTMP*dSIN(X)/GAMMA0
              YPORB(IP)=YY(IP) 
           END DO 
        ELSE
           DO IP=1,NPART                                 !helical undulator
              AWTMP=DSQRT(FAW2(ISTEPZ,XX(IP),YY(IP)))   !aw at particle position
	      XPORB(IP)=XX(IP)-AWTMP*dSIN(X)/GAMMA0
              YPORB(IP)=YY(IP)-AWTMP*dCOS(X)/GAMMA0 
           END DO 
        END IF
      END IF
c
c     Linear interpolation
c     -----------------------------------------------------------
      DO IP=1,NPART  
         IX1=INT((XPORB(IP)+DGRID*XKW0)/DXY)+1            !index in cartesian mesh
         IY1=INT((YPORB(IP)+DGRID*XKW0)/DXY)+1
         IF (XPORB(IP).LT.-DGRID*XKW0) THEN
           IX1=1
           IX2=1
           wxlow=0.d0
           lost=lost+1
           lostid(ip)=1
         ELSE 
           IF (XPORB(IP).GE.DGRID*XKW0) THEN
             IX1=NCAR
             IX2=NCAR
             wxlow=1.d0
             lost=lost+1
             lostid(ip)=1
           ELSE
             IX2=IX1+1
             wxlow=XPORB(IP)+DGRID*XKW0-DXY*float(IX1-1)
             wxlow=1.d0-wxlow/DXY
           END IF
         END IF
         IF (YPORB(IP).LT.-DGRID*XKW0) THEN
           IY1=1
           IY2=1
           wylow=0.d0
           lost=lost+1 
           lostid(ip)=1
         ELSE 
           IF (YPORB(IP).GE.DGRID*XKW0) THEN
             IY1=NCAR
             IY2=NCAR
             wylow=1.d0
             lost=lost+1
             lostid(ip)=1
           ELSE
             IY2=IY1+1
             wylow=YPORB(IP)+DGRID*XKW0-DXY*float(IY1-1)
             wylow=1.d0-wylow/DXY
           END IF
         END IF
         IPOS(1,IP)=(IY1-1)*NCAR+IX1 
         IPOS(2,IP)=(IY2-1)*NCAR+IX1 
         IPOS(3,IP)=(IY1-1)*NCAR+IX2 
         IPOS(4,IP)=(IY2-1)*NCAR+IX2 
         WX(IP)=wxlow
         WY(IP)=wylow 
      End Do       ! IP
c
      RETURN
      END
c


      Subroutine GETPSI(PSI)
c     ==================================================================
c     calculates the total phase PSI as the sum of the radiation phase phi
c     and the particla phase theta.
c     GETPSI is only called by OUTPART to get a non moving bucket.
c     ------------------------------------------------------------------
c
      INCLUDE 'genesis.def'
      INCLUDE 'field.cmn'
      INCLUDE 'input.cmn'
      INCLUDE 'particle.cmn'
c
      REAL*8 philoc,wei,PSI(NPMAX)
      INTEGER ip,idx
      COMPLEX*16 CLOCAL
c
      Do IP=1,NPART
        wei=wx(ip)*wy(iP)
        idx=ipos(1,ip)
        CLOCAL=wei*CRFIELD(idx)
        wei=wx(ip)*(1.d0-wy(ip))
        idx=ipos(2,ip)
        CLOCAL=CLOCAL+wei*CRFIELD(idx)
        wei=(1.d0-wx(ip))*wy(iP)
        idx=ipos(3,ip)
        CLOCAL=CLOCAL+wei*CRFIELD(idx)
        wei=(1.d0-wx(ip))*(1.d0-wy(ip))
        idx=ipos(4,ip)
        CLOCAL=CLOCAL+wei*CRFIELD(idx)
        PHILOC=DATAN2(DIMAG(CLOCAL),DBLE(CLOCAL))
        PSI(IP)=PHILOC+THETA(IP) 
      END DO   !IP
      RETURN  
      END     !GETPSI
